(* OCaml interface: module inteface.
   Copyright (C) 2001-2010 Roberto Bagnara <bagnara@cs.unipr.it>
   Copyright (C) 2010-2012 BUGSENG srl (http://bugseng.com)

This file is part of the Parma Polyhedra Library (PPL).

The PPL is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

The PPL is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1307, USA.

For the most up-to-date information see the Parma Polyhedra Library
site: http://bugseng.com/products/ppl/ . *)

open Gmp

exception PPL_arithmetic_overflow of string
exception PPL_timeout_exception
exception PPL_internal_error of string
exception PPL_unknown_standard_exception of string
exception PPL_unexpected_error of string

type degenerate_element =
    Universe
  | Empty

type linear_expression =
    Variable of int
  | Coefficient of Z.t
  | Unary_Plus of linear_expression
  | Unary_Minus of linear_expression
  | Plus of linear_expression * linear_expression
  | Minus of linear_expression * linear_expression
  | Times of Z.t * linear_expression

type linear_constraint =
    Less_Than of linear_expression * linear_expression
  | Less_Or_Equal of linear_expression * linear_expression
  | Equal of linear_expression * linear_expression
  | Greater_Than of linear_expression * linear_expression
  | Greater_Or_Equal of linear_expression * linear_expression

type linear_generator =
    Line of linear_expression
  | Ray of linear_expression
  | Point of linear_expression * Z.t
  | Closure_Point of linear_expression * Z.t

type linear_grid_generator =
    Grid_Line of linear_expression
  | Grid_Parameter of linear_expression * Z.t
  | Grid_Point of linear_expression * Z.t

type poly_gen_relation =
    Subsumes

type poly_con_relation =
    Is_Disjoint
  | Strictly_Intersects
  | Is_Included
  | Saturates

type relation_with_congruence =
    Is_Disjoint
  | Strictly_Intersects
  | Is_Included

type linear_congruence = linear_expression * linear_expression * Z.t

type constraint_system = linear_constraint list

type generator_system = linear_generator list

type grid_generator_system = linear_grid_generator list

type congruence_system = linear_congruence list

(* Note: the "_RS" suffix is to avoid name clashes with the declaration
   of linear_constraint. *)
type relation_symbol = Less_Than_RS | Less_Or_Equal_RS | Equal_RS
                       | Greater_Than_RS | Greater_Or_Equal_RS

type bounded_integer_type_overflow = Overflow_Wraps
                                     | Overflow_Undefined
                                     | Overflow_Impossible

type bounded_integer_type_representation = Unsigned
                                           | Signed_2_Complement

type bounded_integer_type_width = Bits_8
                                  | Bits_16
                                  | Bits_32
                                  | Bits_64
                                  | Bits_128

type complexity_class = Polynomial_Complexity
                        | Simplex_Complexity
                        | Any_Complexity

type optimization_mode = Minimization | Maximization

type mip_problem_status = Unfeasible_Mip_Problem | Unbounded_Mip_Problem
                        | Optimized_Mip_Problem

type control_parameter_name = Pricing

type control_parameter_value = Pricing_Steepest_Edge_Float
                               | Pricing_Steepest_Edge_Exact
                               | Pricing_Textbook

type pip_problem_status = Unfeasible_Pip_Problem
                        | Optimized_Pip_Problem

type pip_problem_control_parameter_name = Cutting_Strategy | Pivot_Row_Strategy

type pip_problem_control_parameter_value = Cutting_Strategy_First
                                         | Cutting_Strategy_Deepest
                                         | Cutting_Strategy_All
                                         | Pivot_Row_Strategy_First
                                         | Pivot_Row_Strategy_Max_Column

val ppl_version_major:
  unit -> int

val ppl_version_minor:
  unit -> int

val ppl_version_revision:
  unit -> int

val ppl_version_beta:
  unit -> int

val ppl_version:
  unit -> string

val ppl_banner:
  unit -> string

val ppl_io_wrap_string:
  string -> int -> int -> int -> string

val ppl_max_space_dimension:
  unit -> int

val ppl_Coefficient_bits:
  unit -> int

val ppl_Coefficient_is_bounded:
  unit -> bool

val ppl_Coefficient_max:
  unit -> Z.t

val ppl_Coefficient_min:
  unit -> Z.t

val ppl_Linear_Expression_is_zero:
  linear_expression -> bool

val ppl_Linear_Expression_all_homogeneous_terms_are_zero:
  linear_expression -> bool

val ppl_set_rounding_for_PPL:
  unit -> unit

val ppl_restore_pre_PPL_rounding:
  unit -> unit

val ppl_irrational_precision:
  unit -> int

val ppl_set_irrational_precision:
  int -> unit

val ppl_set_timeout:
  int -> unit

val ppl_reset_timeout:
  unit -> unit

val ppl_set_deterministic_timeout:
  int -> int -> unit

val ppl_reset_deterministic_timeout:
  unit -> unit

type mip_problem

val ppl_new_MIP_Problem_from_space_dimension:
  int -> mip_problem

val ppl_new_MIP_Problem:
  int -> constraint_system -> linear_expression
    -> optimization_mode -> mip_problem

val ppl_MIP_Problem_space_dimension:
  mip_problem -> int

val ppl_MIP_Problem_integer_space_dimensions:
  mip_problem -> int list

val ppl_MIP_Problem_constraints:
  mip_problem -> constraint_system

val ppl_MIP_Problem_add_space_dimensions_and_embed:
  mip_problem -> int -> unit

val ppl_MIP_Problem_add_to_integer_space_dimensions:
  mip_problem -> int list -> unit

val ppl_MIP_Problem_add_constraint:
  mip_problem -> linear_constraint -> unit

val ppl_MIP_Problem_add_constraints:
  mip_problem -> constraint_system -> unit

val ppl_MIP_Problem_set_objective_function:
  mip_problem -> linear_expression -> unit

val ppl_MIP_Problem_is_satisfiable:
  mip_problem -> bool

val ppl_MIP_Problem_solve:
  mip_problem -> mip_problem_status

val ppl_MIP_Problem_optimization_mode:
  mip_problem -> optimization_mode

val ppl_MIP_Problem_feasible_point:
  mip_problem -> linear_generator

val ppl_MIP_Problem_optimizing_point:
  mip_problem -> linear_generator

val ppl_MIP_Problem_objective_function:
  mip_problem -> linear_expression

val ppl_MIP_Problem_optimal_value:
  mip_problem -> Z.t * Z.t

val ppl_MIP_Problem_evaluate_objective_function:
  mip_problem -> linear_generator  -> Z.t * Z.t

val ppl_MIP_Problem_OK:
  mip_problem -> bool

val ppl_MIP_Problem_clear:
  mip_problem -> unit

val ppl_MIP_Problem_set_optimization_mode:
  mip_problem -> optimization_mode -> unit

val ppl_MIP_Problem_set_control_parameter:
  mip_problem -> control_parameter_value -> unit

val ppl_MIP_Problem_get_control_parameter:
  mip_problem -> control_parameter_name -> control_parameter_value

val ppl_MIP_Problem_swap:
  mip_problem -> mip_problem -> unit

val ppl_MIP_Problem_ascii_dump:
  mip_problem -> string

type pip_problem

type pip_tree_node

type artificial_parameter = linear_expression * Z.t

val ppl_new_PIP_Problem_from_space_dimension:
  int -> pip_problem

val ppl_new_PIP_Problem:
  int -> constraint_system -> int list -> pip_problem

val ppl_PIP_Problem_space_dimension:
  pip_problem -> int

val ppl_PIP_Problem_parameter_space_dimensions:
  pip_problem -> int list

val ppl_PIP_Problem_constraints:
  pip_problem -> constraint_system

val ppl_PIP_Problem_add_space_dimensions_and_embed:
  pip_problem -> int -> int -> unit

val ppl_PIP_Problem_add_to_parameter_space_dimensions:
  pip_problem -> int list -> unit

val ppl_PIP_Problem_add_constraint:
  pip_problem -> linear_constraint -> unit

val ppl_PIP_Problem_add_constraints:
  pip_problem -> constraint_system -> unit

val ppl_PIP_Problem_is_satisfiable:
  pip_problem -> bool

val ppl_PIP_Problem_solve:
  pip_problem -> pip_problem_status

val ppl_PIP_Problem_solution:
  pip_problem -> pip_tree_node

val ppl_PIP_Problem_optimizing_solution:
  pip_problem -> pip_tree_node

val ppl_PIP_Problem_get_big_parameter_dimension:
  pip_problem -> int

val ppl_PIP_Problem_set_big_parameter_dimension:
  pip_problem -> int -> unit

val ppl_PIP_Problem_has_big_parameter_dimension:
  pip_problem -> bool

val ppl_PIP_Problem_OK:
  pip_problem -> bool

val ppl_PIP_Problem_clear:
  pip_problem -> unit

val ppl_PIP_Problem_set_control_parameter:
  pip_problem -> pip_problem_control_parameter_value -> unit

val ppl_PIP_Problem_get_control_parameter:
  pip_problem -> pip_problem_control_parameter_name
              -> pip_problem_control_parameter_value

val ppl_PIP_Problem_swap:
  pip_problem -> pip_problem -> unit

val ppl_PIP_Problem_ascii_dump:
  pip_problem -> string

val ppl_PIP_Tree_Node_constraints:
  pip_tree_node -> constraint_system

val ppl_PIP_Tree_Node_artificials:
  pip_tree_node -> artificial_parameter list

val ppl_PIP_Tree_Node_ascii_dump:
  pip_tree_node -> string

val ppl_PIP_Tree_Node_OK:
  pip_tree_node -> bool

val ppl_PIP_Tree_Node_is_bottom:
  pip_tree_node -> bool

val ppl_PIP_Tree_Node_is_solution:
  pip_tree_node -> bool

val ppl_PIP_Tree_Node_parametric_values:
  pip_tree_node -> int -> linear_expression

val ppl_PIP_Tree_Node_is_decision:
  pip_tree_node -> bool

val ppl_PIP_Tree_Node_true_child:
  pip_tree_node -> pip_tree_node

val ppl_PIP_Tree_Node_false_child:
  pip_tree_node -> pip_tree_node


type polyhedron
type grid
type rational_box
type bd_shape_mpz_class
type bd_shape_mpq_class
type octagonal_shape_mpz_class
type octagonal_shape_mpq_class
type constraints_product_c_polyhedron_grid
type pointset_powerset_c_polyhedron
type pointset_powerset_nnc_polyhedron
type double_box
type bd_shape_double
type octagonal_shape_double




val ppl_new_C_Polyhedron_from_space_dimension:
  int -> degenerate_element -> polyhedron

val ppl_new_NNC_Polyhedron_from_space_dimension:
  int -> degenerate_element -> polyhedron





val ppl_new_C_Polyhedron_from_C_Polyhedron:
  polyhedron -> polyhedron

val ppl_new_NNC_Polyhedron_from_C_Polyhedron:
  polyhedron -> polyhedron

val ppl_new_C_Polyhedron_from_NNC_Polyhedron:
  polyhedron -> polyhedron

val ppl_new_NNC_Polyhedron_from_NNC_Polyhedron:
  polyhedron -> polyhedron

val ppl_new_C_Polyhedron_from_Grid:
  grid -> polyhedron

val ppl_new_NNC_Polyhedron_from_Grid:
  grid -> polyhedron

val ppl_new_C_Polyhedron_from_Rational_Box:
  rational_box -> polyhedron

val ppl_new_NNC_Polyhedron_from_Rational_Box:
  rational_box -> polyhedron

val ppl_new_C_Polyhedron_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> polyhedron

val ppl_new_C_Polyhedron_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> polyhedron

val ppl_new_C_Polyhedron_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> polyhedron

val ppl_new_C_Polyhedron_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> polyhedron

val ppl_new_C_Polyhedron_from_Double_Box:
  double_box -> polyhedron

val ppl_new_NNC_Polyhedron_from_Double_Box:
  double_box -> polyhedron

val ppl_new_C_Polyhedron_from_BD_Shape_double:
  bd_shape_double -> polyhedron

val ppl_new_NNC_Polyhedron_from_BD_Shape_double:
  bd_shape_double -> polyhedron

val ppl_new_C_Polyhedron_from_Octagonal_Shape_double:
  octagonal_shape_double -> polyhedron

val ppl_new_NNC_Polyhedron_from_Octagonal_Shape_double:
  octagonal_shape_double -> polyhedron






val ppl_new_C_Polyhedron_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_Grid_with_complexity:
  grid -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_Grid_with_complexity:
  grid -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_Double_Box_with_complexity:
  double_box -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_Double_Box_with_complexity:
  double_box -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> polyhedron

val ppl_new_C_Polyhedron_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> polyhedron

val ppl_new_NNC_Polyhedron_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> polyhedron






val ppl_new_C_Polyhedron_from_constraints:
  constraint_system -> polyhedron

val ppl_new_NNC_Polyhedron_from_constraints:
  constraint_system -> polyhedron

val ppl_new_C_Polyhedron_from_congruences:
  congruence_system -> polyhedron

val ppl_new_NNC_Polyhedron_from_congruences:
  congruence_system -> polyhedron

val ppl_new_C_Polyhedron_from_generators:
  generator_system -> polyhedron

val ppl_new_NNC_Polyhedron_from_generators:
  generator_system -> polyhedron






val ppl_Polyhedron_swap:
  polyhedron -> polyhedron -> unit




val ppl_Polyhedron_space_dimension:
  polyhedron -> int
val ppl_Polyhedron_affine_dimension:
  polyhedron -> int




val ppl_Polyhedron_relation_with_constraint:
  polyhedron -> linear_constraint
  -> poly_con_relation list

val ppl_Polyhedron_relation_with_generator:
  polyhedron -> linear_generator
  -> poly_gen_relation list

val ppl_Polyhedron_relation_with_congruence:
  polyhedron -> linear_congruence
  -> poly_con_relation list





val ppl_Polyhedron_get_constraints:
  polyhedron -> constraint_system

val ppl_Polyhedron_get_congruences:
  polyhedron -> congruence_system

val ppl_Polyhedron_get_generators:
  polyhedron -> generator_system





val ppl_Polyhedron_get_minimized_constraints:
  polyhedron -> constraint_system

val ppl_Polyhedron_get_minimized_congruences:
  polyhedron -> congruence_system

val ppl_Polyhedron_get_minimized_generators:
  polyhedron -> generator_system





val ppl_Polyhedron_is_empty:
  polyhedron -> bool

val ppl_Polyhedron_is_universe:
  polyhedron -> bool

val ppl_Polyhedron_is_bounded:
  polyhedron -> bool

val ppl_Polyhedron_contains_integer_point:
  polyhedron -> bool

val ppl_Polyhedron_is_topologically_closed:
  polyhedron -> bool

val ppl_Polyhedron_is_discrete:
  polyhedron -> bool





val ppl_Polyhedron_topological_closure_assign:
  polyhedron -> unit





val ppl_Polyhedron_bounds_from_above:
  polyhedron -> linear_expression -> bool

val ppl_Polyhedron_bounds_from_below:
  polyhedron -> linear_expression -> bool





val ppl_Polyhedron_maximize:
  polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Polyhedron_minimize:
  polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Polyhedron_maximize_with_point:
  polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Polyhedron_minimize_with_point:
  polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Polyhedron_frequency:
  polyhedron -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_Polyhedron_contains_Polyhedron:
  polyhedron -> polyhedron -> bool

val ppl_Polyhedron_strictly_contains_Polyhedron:
  polyhedron -> polyhedron -> bool

val ppl_Polyhedron_is_disjoint_from_Polyhedron:
  polyhedron -> polyhedron -> bool





val ppl_Polyhedron_equals_Polyhedron:
  polyhedron -> polyhedron -> bool




val ppl_Polyhedron_OK:
  polyhedron -> bool



val ppl_Polyhedron_add_constraint:
  polyhedron -> linear_constraint -> unit

val ppl_Polyhedron_add_congruence:
  polyhedron -> linear_congruence -> unit

val ppl_Polyhedron_add_generator:
  polyhedron -> linear_generator -> unit





val ppl_Polyhedron_add_constraints:
  polyhedron -> constraint_system -> unit

val ppl_Polyhedron_add_congruences:
  polyhedron -> congruence_system -> unit

val ppl_Polyhedron_add_generators:
  polyhedron -> generator_system -> unit





val ppl_Polyhedron_refine_with_constraint:
  polyhedron -> linear_constraint -> unit

val ppl_Polyhedron_refine_with_congruence:
  polyhedron -> linear_congruence -> unit





val ppl_Polyhedron_refine_with_constraints:
  polyhedron -> constraint_system -> unit

val ppl_Polyhedron_refine_with_congruences:
  polyhedron -> congruence_system -> unit





val ppl_Polyhedron_intersection_assign:
   polyhedron -> polyhedron -> unit

val ppl_Polyhedron_upper_bound_assign:
   polyhedron -> polyhedron -> unit

val ppl_Polyhedron_difference_assign:
   polyhedron -> polyhedron -> unit

val ppl_Polyhedron_concatenate_assign:
   polyhedron -> polyhedron -> unit

val ppl_Polyhedron_time_elapse_assign:
   polyhedron -> polyhedron -> unit

val ppl_Polyhedron_poly_hull_assign:
   polyhedron -> polyhedron -> unit

val ppl_Polyhedron_poly_difference_assign:
   polyhedron -> polyhedron -> unit





val ppl_Polyhedron_upper_bound_assign_if_exact:
  polyhedron -> polyhedron -> bool

val ppl_Polyhedron_poly_hull_assign_if_exact:
  polyhedron -> polyhedron -> bool





val ppl_Polyhedron_simplify_using_context_assign:
  polyhedron -> polyhedron -> bool




val ppl_Polyhedron_constrains:
  polyhedron -> int -> bool




val ppl_Polyhedron_unconstrain_space_dimension:
  polyhedron -> int -> unit




val ppl_Polyhedron_unconstrain_space_dimensions:
  polyhedron -> int list -> unit




val ppl_Polyhedron_affine_image:
  polyhedron -> int -> linear_expression -> Z.t -> unit

val ppl_Polyhedron_affine_preimage:
  polyhedron -> int -> linear_expression -> Z.t -> unit





val ppl_Polyhedron_bounded_affine_image:
  polyhedron -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Polyhedron_bounded_affine_preimage:
  polyhedron -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Polyhedron_generalized_affine_image:
  polyhedron -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Polyhedron_generalized_affine_preimage:
  polyhedron -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Polyhedron_generalized_affine_image_lhs_rhs:
  polyhedron -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Polyhedron_generalized_affine_preimage_lhs_rhs:
  polyhedron -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Polyhedron_add_space_dimensions_and_embed:
  polyhedron -> int -> unit

val ppl_Polyhedron_add_space_dimensions_and_project:
  polyhedron -> int -> unit





val ppl_Polyhedron_remove_space_dimensions:
  polyhedron -> int list -> unit




val ppl_Polyhedron_remove_higher_space_dimensions:
  polyhedron -> int -> unit




val ppl_Polyhedron_expand_space_dimension:
  polyhedron -> int -> int -> unit



val ppl_Polyhedron_fold_space_dimensions:
  polyhedron -> int list -> int -> unit




val ppl_Polyhedron_map_space_dimensions:
  polyhedron -> (int*int) list -> unit



val ppl_Polyhedron_drop_some_non_integer_points:
  polyhedron -> complexity_class -> unit




val ppl_Polyhedron_drop_some_non_integer_points_2:
  polyhedron -> int list -> complexity_class -> unit




val ppl_Polyhedron_ascii_dump:
  polyhedron -> string




val ppl_Polyhedron_external_memory_in_bytes:
  polyhedron  -> int

val ppl_Polyhedron_total_memory_in_bytes:
  polyhedron  -> int





val ppl_Polyhedron_BHRZ03_widening_assign_with_tokens:
  polyhedron -> polyhedron -> int -> int

val ppl_Polyhedron_H79_widening_assign_with_tokens:
  polyhedron -> polyhedron -> int -> int





val ppl_Polyhedron_BHRZ03_widening_assign:
  polyhedron -> polyhedron -> unit

val ppl_Polyhedron_H79_widening_assign:
  polyhedron -> polyhedron -> unit





val ppl_Polyhedron_widening_assign_with_tokens:
  polyhedron -> polyhedron -> int -> int




val ppl_Polyhedron_widening_assign:
  polyhedron -> polyhedron -> unit




val ppl_Polyhedron_limited_BHRZ03_extrapolation_assign_with_tokens:
  polyhedron -> polyhedron
  -> constraint_system -> int -> int

val ppl_Polyhedron_bounded_BHRZ03_extrapolation_assign_with_tokens:
  polyhedron -> polyhedron
  -> constraint_system -> int -> int

val ppl_Polyhedron_limited_H79_extrapolation_assign_with_tokens:
  polyhedron -> polyhedron
  -> constraint_system -> int -> int

val ppl_Polyhedron_bounded_H79_extrapolation_assign_with_tokens:
  polyhedron -> polyhedron
  -> constraint_system -> int -> int







val ppl_Polyhedron_limited_BHRZ03_extrapolation_assign:
  polyhedron -> polyhedron
  -> constraint_system -> unit

val ppl_Polyhedron_bounded_BHRZ03_extrapolation_assign:
  polyhedron -> polyhedron
  -> constraint_system -> unit

val ppl_Polyhedron_limited_H79_extrapolation_assign:
  polyhedron -> polyhedron
  -> constraint_system -> unit

val ppl_Polyhedron_bounded_H79_extrapolation_assign:
  polyhedron -> polyhedron
  -> constraint_system -> unit











val ppl_Polyhedron_linear_partition:
  polyhedron -> polyhedron -> polyhedron * pointset_powerset_nnc_polyhedron





val ppl_Polyhedron_wrap_assign: polyhedron
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_C_Polyhedron:
  polyhedron -> bool

val ppl_termination_test_PR_C_Polyhedron:
  polyhedron -> bool

val ppl_termination_test_MS_NNC_Polyhedron:
  polyhedron -> bool

val ppl_termination_test_PR_NNC_Polyhedron:
  polyhedron -> bool






val ppl_one_affine_ranking_function_MS_C_Polyhedron:
  polyhedron -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_C_Polyhedron:
  polyhedron -> bool * linear_generator

val ppl_one_affine_ranking_function_MS_NNC_Polyhedron:
  polyhedron -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_NNC_Polyhedron:
  polyhedron -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_C_Polyhedron:
  polyhedron -> polyhedron

val ppl_all_affine_ranking_functions_PR_C_Polyhedron:
  polyhedron -> polyhedron

val ppl_all_affine_ranking_functions_MS_NNC_Polyhedron:
  polyhedron -> polyhedron

val ppl_all_affine_ranking_functions_PR_NNC_Polyhedron:
  polyhedron -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_C_Polyhedron:
  polyhedron -> polyhedron * polyhedron

val ppl_all_affine_quasi_ranking_functions_MS_NNC_Polyhedron:
  polyhedron -> polyhedron * polyhedron





val ppl_termination_test_MS_C_Polyhedron_2:
  polyhedron -> polyhedron -> bool

val ppl_termination_test_PR_C_Polyhedron_2:
  polyhedron -> polyhedron -> bool

val ppl_termination_test_MS_NNC_Polyhedron_2:
  polyhedron -> polyhedron -> bool

val ppl_termination_test_PR_NNC_Polyhedron_2:
  polyhedron -> polyhedron -> bool






val ppl_one_affine_ranking_function_MS_C_Polyhedron_2:
  polyhedron -> polyhedron -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_C_Polyhedron_2:
  polyhedron -> polyhedron -> bool * linear_generator

val ppl_one_affine_ranking_function_MS_NNC_Polyhedron_2:
  polyhedron -> polyhedron -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_NNC_Polyhedron_2:
  polyhedron -> polyhedron -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_C_Polyhedron_2:
  polyhedron -> polyhedron -> polyhedron

val ppl_all_affine_ranking_functions_PR_C_Polyhedron_2:
  polyhedron -> polyhedron -> polyhedron

val ppl_all_affine_ranking_functions_MS_NNC_Polyhedron_2:
  polyhedron -> polyhedron -> polyhedron

val ppl_all_affine_ranking_functions_PR_NNC_Polyhedron_2:
  polyhedron -> polyhedron -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_C_Polyhedron_2:
  polyhedron -> polyhedron -> polyhedron * polyhedron

val ppl_all_affine_quasi_ranking_functions_MS_NNC_Polyhedron_2:
  polyhedron -> polyhedron -> polyhedron * polyhedron








val ppl_new_Grid_from_space_dimension:
  int -> degenerate_element -> grid





val ppl_new_Grid_from_C_Polyhedron:
  polyhedron -> grid

val ppl_new_Grid_from_NNC_Polyhedron:
  polyhedron -> grid

val ppl_new_Grid_from_Grid:
  grid -> grid

val ppl_new_Grid_from_Rational_Box:
  rational_box -> grid

val ppl_new_Grid_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> grid

val ppl_new_Grid_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> grid

val ppl_new_Grid_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> grid

val ppl_new_Grid_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> grid

val ppl_new_Grid_from_Double_Box:
  double_box -> grid

val ppl_new_Grid_from_BD_Shape_double:
  bd_shape_double -> grid

val ppl_new_Grid_from_Octagonal_Shape_double:
  octagonal_shape_double -> grid






val ppl_new_Grid_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> grid

val ppl_new_Grid_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> grid

val ppl_new_Grid_from_Grid_with_complexity:
  grid -> complexity_class -> grid

val ppl_new_Grid_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> grid

val ppl_new_Grid_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> grid

val ppl_new_Grid_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> grid

val ppl_new_Grid_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> grid

val ppl_new_Grid_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> grid

val ppl_new_Grid_from_Double_Box_with_complexity:
  double_box -> complexity_class -> grid

val ppl_new_Grid_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> grid

val ppl_new_Grid_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> grid






val ppl_new_Grid_from_constraints:
  constraint_system -> grid

val ppl_new_Grid_from_congruences:
  congruence_system -> grid

val ppl_new_Grid_from_grid_generators:
  grid_generator_system -> grid






val ppl_Grid_swap:
  grid -> grid -> unit




val ppl_Grid_space_dimension:
  grid -> int
val ppl_Grid_affine_dimension:
  grid -> int




val ppl_Grid_relation_with_constraint:
  grid -> linear_constraint
  -> poly_con_relation list

val ppl_Grid_relation_with_generator:
  grid -> linear_generator
  -> poly_gen_relation list

val ppl_Grid_relation_with_congruence:
  grid -> linear_congruence
  -> poly_con_relation list

val ppl_Grid_relation_with_grid_generator:
  grid -> linear_grid_generator
  -> poly_gen_relation list





val ppl_Grid_get_constraints:
  grid -> constraint_system

val ppl_Grid_get_congruences:
  grid -> congruence_system

val ppl_Grid_get_grid_generators:
  grid -> grid_generator_system





val ppl_Grid_get_minimized_constraints:
  grid -> constraint_system

val ppl_Grid_get_minimized_congruences:
  grid -> congruence_system

val ppl_Grid_get_minimized_grid_generators:
  grid -> grid_generator_system





val ppl_Grid_is_empty:
  grid -> bool

val ppl_Grid_is_universe:
  grid -> bool

val ppl_Grid_is_bounded:
  grid -> bool

val ppl_Grid_contains_integer_point:
  grid -> bool

val ppl_Grid_is_topologically_closed:
  grid -> bool

val ppl_Grid_is_discrete:
  grid -> bool





val ppl_Grid_topological_closure_assign:
  grid -> unit





val ppl_Grid_bounds_from_above:
  grid -> linear_expression -> bool

val ppl_Grid_bounds_from_below:
  grid -> linear_expression -> bool





val ppl_Grid_maximize:
  grid -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Grid_minimize:
  grid -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Grid_maximize_with_point:
  grid -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Grid_minimize_with_point:
  grid -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Grid_frequency:
  grid -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_Grid_contains_Grid:
  grid -> grid -> bool

val ppl_Grid_strictly_contains_Grid:
  grid -> grid -> bool

val ppl_Grid_is_disjoint_from_Grid:
  grid -> grid -> bool





val ppl_Grid_equals_Grid:
  grid -> grid -> bool




val ppl_Grid_OK:
  grid -> bool



val ppl_Grid_add_constraint:
  grid -> linear_constraint -> unit

val ppl_Grid_add_congruence:
  grid -> linear_congruence -> unit

val ppl_Grid_add_grid_generator:
  grid -> linear_grid_generator -> unit





val ppl_Grid_add_constraints:
  grid -> constraint_system -> unit

val ppl_Grid_add_congruences:
  grid -> congruence_system -> unit

val ppl_Grid_add_grid_generators:
  grid -> grid_generator_system -> unit





val ppl_Grid_refine_with_constraint:
  grid -> linear_constraint -> unit

val ppl_Grid_refine_with_congruence:
  grid -> linear_congruence -> unit





val ppl_Grid_refine_with_constraints:
  grid -> constraint_system -> unit

val ppl_Grid_refine_with_congruences:
  grid -> congruence_system -> unit





val ppl_Grid_intersection_assign:
   grid -> grid -> unit

val ppl_Grid_upper_bound_assign:
   grid -> grid -> unit

val ppl_Grid_difference_assign:
   grid -> grid -> unit

val ppl_Grid_concatenate_assign:
   grid -> grid -> unit

val ppl_Grid_time_elapse_assign:
   grid -> grid -> unit





val ppl_Grid_upper_bound_assign_if_exact:
  grid -> grid -> bool





val ppl_Grid_simplify_using_context_assign:
  grid -> grid -> bool




val ppl_Grid_constrains:
  grid -> int -> bool




val ppl_Grid_unconstrain_space_dimension:
  grid -> int -> unit




val ppl_Grid_unconstrain_space_dimensions:
  grid -> int list -> unit




val ppl_Grid_affine_image:
  grid -> int -> linear_expression -> Z.t -> unit

val ppl_Grid_affine_preimage:
  grid -> int -> linear_expression -> Z.t -> unit





val ppl_Grid_bounded_affine_image:
  grid -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Grid_bounded_affine_preimage:
  grid -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Grid_generalized_affine_image:
  grid -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Grid_generalized_affine_preimage:
  grid -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Grid_generalized_affine_image_lhs_rhs:
  grid -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Grid_generalized_affine_preimage_lhs_rhs:
  grid -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Grid_generalized_affine_image_with_congruence:
  grid -> int -> relation_symbol -> linear_expression
  -> Z.t -> Z.t -> unit

val ppl_Grid_generalized_affine_preimage_with_congruence:
  grid -> int -> relation_symbol -> linear_expression
  -> Z.t -> Z.t -> unit





val ppl_Grid_generalized_affine_image_lhs_rhs_with_congruence:
  grid -> linear_expression
  -> relation_symbol -> linear_expression -> Z.t
  -> unit

val ppl_Grid_generalized_affine_preimage_lhs_rhs_with_congruence:
  grid -> linear_expression
  -> relation_symbol -> linear_expression -> Z.t
  -> unit





val ppl_Grid_add_space_dimensions_and_embed:
  grid -> int -> unit

val ppl_Grid_add_space_dimensions_and_project:
  grid -> int -> unit





val ppl_Grid_remove_space_dimensions:
  grid -> int list -> unit




val ppl_Grid_remove_higher_space_dimensions:
  grid -> int -> unit




val ppl_Grid_expand_space_dimension:
  grid -> int -> int -> unit



val ppl_Grid_fold_space_dimensions:
  grid -> int list -> int -> unit




val ppl_Grid_map_space_dimensions:
  grid -> (int*int) list -> unit



val ppl_Grid_drop_some_non_integer_points:
  grid -> complexity_class -> unit




val ppl_Grid_drop_some_non_integer_points_2:
  grid -> int list -> complexity_class -> unit




val ppl_Grid_ascii_dump:
  grid -> string




val ppl_Grid_external_memory_in_bytes:
  grid  -> int

val ppl_Grid_total_memory_in_bytes:
  grid  -> int





val ppl_Grid_congruence_widening_assign_with_tokens:
  grid -> grid -> int -> int

val ppl_Grid_generator_widening_assign_with_tokens:
  grid -> grid -> int -> int





val ppl_Grid_congruence_widening_assign:
  grid -> grid -> unit

val ppl_Grid_generator_widening_assign:
  grid -> grid -> unit





val ppl_Grid_widening_assign_with_tokens:
  grid -> grid -> int -> int




val ppl_Grid_widening_assign:
  grid -> grid -> unit




val ppl_Grid_limited_congruence_extrapolation_assign_with_tokens:
  grid -> grid
  -> congruence_system -> int -> int

val ppl_Grid_limited_generator_extrapolation_assign_with_tokens:
  grid -> grid
  -> congruence_system -> int -> int







val ppl_Grid_limited_congruence_extrapolation_assign:
  grid -> grid
  -> congruence_system -> unit

val ppl_Grid_limited_generator_extrapolation_assign:
  grid -> grid
  -> congruence_system -> unit











val ppl_Grid_wrap_assign: grid
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_Grid:
  grid -> bool

val ppl_termination_test_PR_Grid:
  grid -> bool






val ppl_one_affine_ranking_function_MS_Grid:
  grid -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Grid:
  grid -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Grid:
  grid -> polyhedron

val ppl_all_affine_ranking_functions_PR_Grid:
  grid -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Grid:
  grid -> polyhedron * polyhedron





val ppl_termination_test_MS_Grid_2:
  grid -> grid -> bool

val ppl_termination_test_PR_Grid_2:
  grid -> grid -> bool






val ppl_one_affine_ranking_function_MS_Grid_2:
  grid -> grid -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Grid_2:
  grid -> grid -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Grid_2:
  grid -> grid -> polyhedron

val ppl_all_affine_ranking_functions_PR_Grid_2:
  grid -> grid -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Grid_2:
  grid -> grid -> polyhedron * polyhedron








val ppl_new_Rational_Box_from_space_dimension:
  int -> degenerate_element -> rational_box





val ppl_new_Rational_Box_from_C_Polyhedron:
  polyhedron -> rational_box

val ppl_new_Rational_Box_from_NNC_Polyhedron:
  polyhedron -> rational_box

val ppl_new_Rational_Box_from_Grid:
  grid -> rational_box

val ppl_new_Rational_Box_from_Rational_Box:
  rational_box -> rational_box

val ppl_new_Rational_Box_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> rational_box

val ppl_new_Rational_Box_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> rational_box

val ppl_new_Rational_Box_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> rational_box

val ppl_new_Rational_Box_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> rational_box

val ppl_new_Rational_Box_from_Double_Box:
  double_box -> rational_box

val ppl_new_Rational_Box_from_BD_Shape_double:
  bd_shape_double -> rational_box

val ppl_new_Rational_Box_from_Octagonal_Shape_double:
  octagonal_shape_double -> rational_box






val ppl_new_Rational_Box_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_Grid_with_complexity:
  grid -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_Double_Box_with_complexity:
  double_box -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> rational_box

val ppl_new_Rational_Box_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> rational_box






val ppl_new_Rational_Box_from_constraints:
  constraint_system -> rational_box

val ppl_new_Rational_Box_from_congruences:
  congruence_system -> rational_box

val ppl_new_Rational_Box_from_generators:
  generator_system -> rational_box






val ppl_Rational_Box_swap:
  rational_box -> rational_box -> unit




val ppl_Rational_Box_space_dimension:
  rational_box -> int
val ppl_Rational_Box_affine_dimension:
  rational_box -> int




val ppl_Rational_Box_relation_with_constraint:
  rational_box -> linear_constraint
  -> poly_con_relation list

val ppl_Rational_Box_relation_with_generator:
  rational_box -> linear_generator
  -> poly_gen_relation list

val ppl_Rational_Box_relation_with_congruence:
  rational_box -> linear_congruence
  -> poly_con_relation list





val ppl_Rational_Box_get_constraints:
  rational_box -> constraint_system

val ppl_Rational_Box_get_congruences:
  rational_box -> congruence_system





val ppl_Rational_Box_get_minimized_constraints:
  rational_box -> constraint_system

val ppl_Rational_Box_get_minimized_congruences:
  rational_box -> congruence_system





val ppl_Rational_Box_is_empty:
  rational_box -> bool

val ppl_Rational_Box_is_universe:
  rational_box -> bool

val ppl_Rational_Box_is_bounded:
  rational_box -> bool

val ppl_Rational_Box_contains_integer_point:
  rational_box -> bool

val ppl_Rational_Box_is_topologically_closed:
  rational_box -> bool

val ppl_Rational_Box_is_discrete:
  rational_box -> bool





val ppl_Rational_Box_topological_closure_assign:
  rational_box -> unit





val ppl_Rational_Box_bounds_from_above:
  rational_box -> linear_expression -> bool

val ppl_Rational_Box_bounds_from_below:
  rational_box -> linear_expression -> bool





val ppl_Rational_Box_has_upper_bound:
  rational_box -> int
  -> bool * Z.t * Z.t * bool

val ppl_Rational_Box_has_lower_bound:
  rational_box -> int
  -> bool * Z.t * Z.t * bool





val ppl_Rational_Box_maximize:
  rational_box -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Rational_Box_minimize:
  rational_box -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Rational_Box_maximize_with_point:
  rational_box -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Rational_Box_minimize_with_point:
  rational_box -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Rational_Box_frequency:
  rational_box -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_Rational_Box_contains_Rational_Box:
  rational_box -> rational_box -> bool

val ppl_Rational_Box_strictly_contains_Rational_Box:
  rational_box -> rational_box -> bool

val ppl_Rational_Box_is_disjoint_from_Rational_Box:
  rational_box -> rational_box -> bool





val ppl_Rational_Box_equals_Rational_Box:
  rational_box -> rational_box -> bool




val ppl_Rational_Box_OK:
  rational_box -> bool



val ppl_Rational_Box_add_constraint:
  rational_box -> linear_constraint -> unit

val ppl_Rational_Box_add_congruence:
  rational_box -> linear_congruence -> unit





val ppl_Rational_Box_add_constraints:
  rational_box -> constraint_system -> unit

val ppl_Rational_Box_add_congruences:
  rational_box -> congruence_system -> unit





val ppl_Rational_Box_refine_with_constraint:
  rational_box -> linear_constraint -> unit

val ppl_Rational_Box_refine_with_congruence:
  rational_box -> linear_congruence -> unit





val ppl_Rational_Box_refine_with_constraints:
  rational_box -> constraint_system -> unit

val ppl_Rational_Box_refine_with_congruences:
  rational_box -> congruence_system -> unit





val ppl_Rational_Box_intersection_assign:
   rational_box -> rational_box -> unit

val ppl_Rational_Box_upper_bound_assign:
   rational_box -> rational_box -> unit

val ppl_Rational_Box_difference_assign:
   rational_box -> rational_box -> unit

val ppl_Rational_Box_concatenate_assign:
   rational_box -> rational_box -> unit

val ppl_Rational_Box_time_elapse_assign:
   rational_box -> rational_box -> unit





val ppl_Rational_Box_upper_bound_assign_if_exact:
  rational_box -> rational_box -> bool





val ppl_Rational_Box_simplify_using_context_assign:
  rational_box -> rational_box -> bool




val ppl_Rational_Box_constrains:
  rational_box -> int -> bool




val ppl_Rational_Box_unconstrain_space_dimension:
  rational_box -> int -> unit




val ppl_Rational_Box_unconstrain_space_dimensions:
  rational_box -> int list -> unit




val ppl_Rational_Box_affine_image:
  rational_box -> int -> linear_expression -> Z.t -> unit

val ppl_Rational_Box_affine_preimage:
  rational_box -> int -> linear_expression -> Z.t -> unit





val ppl_Rational_Box_bounded_affine_image:
  rational_box -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Rational_Box_bounded_affine_preimage:
  rational_box -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Rational_Box_generalized_affine_image:
  rational_box -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Rational_Box_generalized_affine_preimage:
  rational_box -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Rational_Box_generalized_affine_image_lhs_rhs:
  rational_box -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Rational_Box_generalized_affine_preimage_lhs_rhs:
  rational_box -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Rational_Box_add_space_dimensions_and_embed:
  rational_box -> int -> unit

val ppl_Rational_Box_add_space_dimensions_and_project:
  rational_box -> int -> unit





val ppl_Rational_Box_remove_space_dimensions:
  rational_box -> int list -> unit




val ppl_Rational_Box_remove_higher_space_dimensions:
  rational_box -> int -> unit




val ppl_Rational_Box_expand_space_dimension:
  rational_box -> int -> int -> unit



val ppl_Rational_Box_fold_space_dimensions:
  rational_box -> int list -> int -> unit




val ppl_Rational_Box_map_space_dimensions:
  rational_box -> (int*int) list -> unit



val ppl_Rational_Box_drop_some_non_integer_points:
  rational_box -> complexity_class -> unit




val ppl_Rational_Box_drop_some_non_integer_points_2:
  rational_box -> int list -> complexity_class -> unit




val ppl_Rational_Box_ascii_dump:
  rational_box -> string




val ppl_Rational_Box_external_memory_in_bytes:
  rational_box  -> int

val ppl_Rational_Box_total_memory_in_bytes:
  rational_box  -> int





val ppl_Rational_Box_CC76_widening_assign_with_tokens:
  rational_box -> rational_box -> int -> int





val ppl_Rational_Box_CC76_widening_assign:
  rational_box -> rational_box -> unit





val ppl_Rational_Box_widening_assign_with_tokens:
  rational_box -> rational_box -> int -> int




val ppl_Rational_Box_widening_assign:
  rational_box -> rational_box -> unit




val ppl_Rational_Box_limited_CC76_extrapolation_assign_with_tokens:
  rational_box -> rational_box
  -> constraint_system -> int -> int







val ppl_Rational_Box_limited_CC76_extrapolation_assign:
  rational_box -> rational_box
  -> constraint_system -> unit







val ppl_Rational_Box_CC76_narrowing_assign:
  rational_box  -> rational_box -> unit





val ppl_Rational_Box_linear_partition:
  rational_box -> rational_box -> rational_box * pointset_powerset_nnc_polyhedron





val ppl_Rational_Box_wrap_assign: rational_box
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_Rational_Box:
  rational_box -> bool

val ppl_termination_test_PR_Rational_Box:
  rational_box -> bool






val ppl_one_affine_ranking_function_MS_Rational_Box:
  rational_box -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Rational_Box:
  rational_box -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Rational_Box:
  rational_box -> polyhedron

val ppl_all_affine_ranking_functions_PR_Rational_Box:
  rational_box -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Rational_Box:
  rational_box -> polyhedron * polyhedron





val ppl_termination_test_MS_Rational_Box_2:
  rational_box -> rational_box -> bool

val ppl_termination_test_PR_Rational_Box_2:
  rational_box -> rational_box -> bool






val ppl_one_affine_ranking_function_MS_Rational_Box_2:
  rational_box -> rational_box -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Rational_Box_2:
  rational_box -> rational_box -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Rational_Box_2:
  rational_box -> rational_box -> polyhedron

val ppl_all_affine_ranking_functions_PR_Rational_Box_2:
  rational_box -> rational_box -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Rational_Box_2:
  rational_box -> rational_box -> polyhedron * polyhedron








val ppl_new_BD_Shape_mpz_class_from_space_dimension:
  int -> degenerate_element -> bd_shape_mpz_class





val ppl_new_BD_Shape_mpz_class_from_C_Polyhedron:
  polyhedron -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_NNC_Polyhedron:
  polyhedron -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Grid:
  grid -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Rational_Box:
  rational_box -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Double_Box:
  double_box -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_BD_Shape_double:
  bd_shape_double -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Octagonal_Shape_double:
  octagonal_shape_double -> bd_shape_mpz_class






val ppl_new_BD_Shape_mpz_class_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Grid_with_complexity:
  grid -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Double_Box_with_complexity:
  double_box -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> bd_shape_mpz_class






val ppl_new_BD_Shape_mpz_class_from_constraints:
  constraint_system -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_congruences:
  congruence_system -> bd_shape_mpz_class

val ppl_new_BD_Shape_mpz_class_from_generators:
  generator_system -> bd_shape_mpz_class






val ppl_BD_Shape_mpz_class_swap:
  bd_shape_mpz_class -> bd_shape_mpz_class -> unit




val ppl_BD_Shape_mpz_class_space_dimension:
  bd_shape_mpz_class -> int
val ppl_BD_Shape_mpz_class_affine_dimension:
  bd_shape_mpz_class -> int




val ppl_BD_Shape_mpz_class_relation_with_constraint:
  bd_shape_mpz_class -> linear_constraint
  -> poly_con_relation list

val ppl_BD_Shape_mpz_class_relation_with_generator:
  bd_shape_mpz_class -> linear_generator
  -> poly_gen_relation list

val ppl_BD_Shape_mpz_class_relation_with_congruence:
  bd_shape_mpz_class -> linear_congruence
  -> poly_con_relation list





val ppl_BD_Shape_mpz_class_get_constraints:
  bd_shape_mpz_class -> constraint_system

val ppl_BD_Shape_mpz_class_get_congruences:
  bd_shape_mpz_class -> congruence_system





val ppl_BD_Shape_mpz_class_get_minimized_constraints:
  bd_shape_mpz_class -> constraint_system

val ppl_BD_Shape_mpz_class_get_minimized_congruences:
  bd_shape_mpz_class -> congruence_system





val ppl_BD_Shape_mpz_class_is_empty:
  bd_shape_mpz_class -> bool

val ppl_BD_Shape_mpz_class_is_universe:
  bd_shape_mpz_class -> bool

val ppl_BD_Shape_mpz_class_is_bounded:
  bd_shape_mpz_class -> bool

val ppl_BD_Shape_mpz_class_contains_integer_point:
  bd_shape_mpz_class -> bool

val ppl_BD_Shape_mpz_class_is_topologically_closed:
  bd_shape_mpz_class -> bool

val ppl_BD_Shape_mpz_class_is_discrete:
  bd_shape_mpz_class -> bool





val ppl_BD_Shape_mpz_class_topological_closure_assign:
  bd_shape_mpz_class -> unit





val ppl_BD_Shape_mpz_class_bounds_from_above:
  bd_shape_mpz_class -> linear_expression -> bool

val ppl_BD_Shape_mpz_class_bounds_from_below:
  bd_shape_mpz_class -> linear_expression -> bool





val ppl_BD_Shape_mpz_class_maximize:
  bd_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_BD_Shape_mpz_class_minimize:
  bd_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_BD_Shape_mpz_class_maximize_with_point:
  bd_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_BD_Shape_mpz_class_minimize_with_point:
  bd_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_BD_Shape_mpz_class_frequency:
  bd_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_BD_Shape_mpz_class_contains_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool

val ppl_BD_Shape_mpz_class_strictly_contains_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool

val ppl_BD_Shape_mpz_class_is_disjoint_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool





val ppl_BD_Shape_mpz_class_equals_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool




val ppl_BD_Shape_mpz_class_OK:
  bd_shape_mpz_class -> bool



val ppl_BD_Shape_mpz_class_add_constraint:
  bd_shape_mpz_class -> linear_constraint -> unit

val ppl_BD_Shape_mpz_class_add_congruence:
  bd_shape_mpz_class -> linear_congruence -> unit





val ppl_BD_Shape_mpz_class_add_constraints:
  bd_shape_mpz_class -> constraint_system -> unit

val ppl_BD_Shape_mpz_class_add_congruences:
  bd_shape_mpz_class -> congruence_system -> unit





val ppl_BD_Shape_mpz_class_refine_with_constraint:
  bd_shape_mpz_class -> linear_constraint -> unit

val ppl_BD_Shape_mpz_class_refine_with_congruence:
  bd_shape_mpz_class -> linear_congruence -> unit





val ppl_BD_Shape_mpz_class_refine_with_constraints:
  bd_shape_mpz_class -> constraint_system -> unit

val ppl_BD_Shape_mpz_class_refine_with_congruences:
  bd_shape_mpz_class -> congruence_system -> unit





val ppl_BD_Shape_mpz_class_intersection_assign:
   bd_shape_mpz_class -> bd_shape_mpz_class -> unit

val ppl_BD_Shape_mpz_class_upper_bound_assign:
   bd_shape_mpz_class -> bd_shape_mpz_class -> unit

val ppl_BD_Shape_mpz_class_difference_assign:
   bd_shape_mpz_class -> bd_shape_mpz_class -> unit

val ppl_BD_Shape_mpz_class_concatenate_assign:
   bd_shape_mpz_class -> bd_shape_mpz_class -> unit

val ppl_BD_Shape_mpz_class_time_elapse_assign:
   bd_shape_mpz_class -> bd_shape_mpz_class -> unit





val ppl_BD_Shape_mpz_class_upper_bound_assign_if_exact:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool





val ppl_BD_Shape_mpz_class_simplify_using_context_assign:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool




val ppl_BD_Shape_mpz_class_constrains:
  bd_shape_mpz_class -> int -> bool




val ppl_BD_Shape_mpz_class_unconstrain_space_dimension:
  bd_shape_mpz_class -> int -> unit




val ppl_BD_Shape_mpz_class_unconstrain_space_dimensions:
  bd_shape_mpz_class -> int list -> unit




val ppl_BD_Shape_mpz_class_affine_image:
  bd_shape_mpz_class -> int -> linear_expression -> Z.t -> unit

val ppl_BD_Shape_mpz_class_affine_preimage:
  bd_shape_mpz_class -> int -> linear_expression -> Z.t -> unit





val ppl_BD_Shape_mpz_class_bounded_affine_image:
  bd_shape_mpz_class -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_BD_Shape_mpz_class_bounded_affine_preimage:
  bd_shape_mpz_class -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_BD_Shape_mpz_class_generalized_affine_image:
  bd_shape_mpz_class -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_BD_Shape_mpz_class_generalized_affine_preimage:
  bd_shape_mpz_class -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_BD_Shape_mpz_class_generalized_affine_image_lhs_rhs:
  bd_shape_mpz_class -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_BD_Shape_mpz_class_generalized_affine_preimage_lhs_rhs:
  bd_shape_mpz_class -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_BD_Shape_mpz_class_add_space_dimensions_and_embed:
  bd_shape_mpz_class -> int -> unit

val ppl_BD_Shape_mpz_class_add_space_dimensions_and_project:
  bd_shape_mpz_class -> int -> unit





val ppl_BD_Shape_mpz_class_remove_space_dimensions:
  bd_shape_mpz_class -> int list -> unit




val ppl_BD_Shape_mpz_class_remove_higher_space_dimensions:
  bd_shape_mpz_class -> int -> unit




val ppl_BD_Shape_mpz_class_expand_space_dimension:
  bd_shape_mpz_class -> int -> int -> unit



val ppl_BD_Shape_mpz_class_fold_space_dimensions:
  bd_shape_mpz_class -> int list -> int -> unit




val ppl_BD_Shape_mpz_class_map_space_dimensions:
  bd_shape_mpz_class -> (int*int) list -> unit



val ppl_BD_Shape_mpz_class_drop_some_non_integer_points:
  bd_shape_mpz_class -> complexity_class -> unit




val ppl_BD_Shape_mpz_class_drop_some_non_integer_points_2:
  bd_shape_mpz_class -> int list -> complexity_class -> unit




val ppl_BD_Shape_mpz_class_ascii_dump:
  bd_shape_mpz_class -> string




val ppl_BD_Shape_mpz_class_external_memory_in_bytes:
  bd_shape_mpz_class  -> int

val ppl_BD_Shape_mpz_class_total_memory_in_bytes:
  bd_shape_mpz_class  -> int





val ppl_BD_Shape_mpz_class_BHMZ05_widening_assign_with_tokens:
  bd_shape_mpz_class -> bd_shape_mpz_class -> int -> int

val ppl_BD_Shape_mpz_class_H79_widening_assign_with_tokens:
  bd_shape_mpz_class -> bd_shape_mpz_class -> int -> int





val ppl_BD_Shape_mpz_class_BHMZ05_widening_assign:
  bd_shape_mpz_class -> bd_shape_mpz_class -> unit

val ppl_BD_Shape_mpz_class_H79_widening_assign:
  bd_shape_mpz_class -> bd_shape_mpz_class -> unit





val ppl_BD_Shape_mpz_class_widening_assign_with_tokens:
  bd_shape_mpz_class -> bd_shape_mpz_class -> int -> int




val ppl_BD_Shape_mpz_class_widening_assign:
  bd_shape_mpz_class -> bd_shape_mpz_class -> unit




val ppl_BD_Shape_mpz_class_limited_BHMZ05_extrapolation_assign_with_tokens:
  bd_shape_mpz_class -> bd_shape_mpz_class
  -> constraint_system -> int -> int

val ppl_BD_Shape_mpz_class_limited_H79_extrapolation_assign_with_tokens:
  bd_shape_mpz_class -> bd_shape_mpz_class
  -> constraint_system -> int -> int

val ppl_BD_Shape_mpz_class_limited_CC76_extrapolation_assign_with_tokens:
  bd_shape_mpz_class -> bd_shape_mpz_class
  -> constraint_system -> int -> int







val ppl_BD_Shape_mpz_class_limited_BHMZ05_extrapolation_assign:
  bd_shape_mpz_class -> bd_shape_mpz_class
  -> constraint_system -> unit

val ppl_BD_Shape_mpz_class_limited_H79_extrapolation_assign:
  bd_shape_mpz_class -> bd_shape_mpz_class
  -> constraint_system -> unit

val ppl_BD_Shape_mpz_class_limited_CC76_extrapolation_assign:
  bd_shape_mpz_class -> bd_shape_mpz_class
  -> constraint_system -> unit







val ppl_BD_Shape_mpz_class_CC76_extrapolation_assign_with_tokens:
  bd_shape_mpz_class  -> bd_shape_mpz_class -> int -> int





val ppl_BD_Shape_mpz_class_CC76_extrapolation_assign:
  bd_shape_mpz_class  -> bd_shape_mpz_class -> unit





val ppl_BD_Shape_mpz_class_CC76_narrowing_assign:
  bd_shape_mpz_class  -> bd_shape_mpz_class -> unit





val ppl_BD_Shape_mpz_class_linear_partition:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bd_shape_mpz_class * pointset_powerset_nnc_polyhedron





val ppl_BD_Shape_mpz_class_wrap_assign: bd_shape_mpz_class
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bool

val ppl_termination_test_PR_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bool






val ppl_one_affine_ranking_function_MS_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_BD_Shape_mpz_class:
  bd_shape_mpz_class -> polyhedron

val ppl_all_affine_ranking_functions_PR_BD_Shape_mpz_class:
  bd_shape_mpz_class -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_BD_Shape_mpz_class:
  bd_shape_mpz_class -> polyhedron * polyhedron





val ppl_termination_test_MS_BD_Shape_mpz_class_2:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool

val ppl_termination_test_PR_BD_Shape_mpz_class_2:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool






val ppl_one_affine_ranking_function_MS_BD_Shape_mpz_class_2:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_BD_Shape_mpz_class_2:
  bd_shape_mpz_class -> bd_shape_mpz_class -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_BD_Shape_mpz_class_2:
  bd_shape_mpz_class -> bd_shape_mpz_class -> polyhedron

val ppl_all_affine_ranking_functions_PR_BD_Shape_mpz_class_2:
  bd_shape_mpz_class -> bd_shape_mpz_class -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_BD_Shape_mpz_class_2:
  bd_shape_mpz_class -> bd_shape_mpz_class -> polyhedron * polyhedron








val ppl_new_BD_Shape_mpq_class_from_space_dimension:
  int -> degenerate_element -> bd_shape_mpq_class





val ppl_new_BD_Shape_mpq_class_from_C_Polyhedron:
  polyhedron -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_NNC_Polyhedron:
  polyhedron -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Grid:
  grid -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Rational_Box:
  rational_box -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Double_Box:
  double_box -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_BD_Shape_double:
  bd_shape_double -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Octagonal_Shape_double:
  octagonal_shape_double -> bd_shape_mpq_class






val ppl_new_BD_Shape_mpq_class_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Grid_with_complexity:
  grid -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Double_Box_with_complexity:
  double_box -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> bd_shape_mpq_class






val ppl_new_BD_Shape_mpq_class_from_constraints:
  constraint_system -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_congruences:
  congruence_system -> bd_shape_mpq_class

val ppl_new_BD_Shape_mpq_class_from_generators:
  generator_system -> bd_shape_mpq_class






val ppl_BD_Shape_mpq_class_swap:
  bd_shape_mpq_class -> bd_shape_mpq_class -> unit




val ppl_BD_Shape_mpq_class_space_dimension:
  bd_shape_mpq_class -> int
val ppl_BD_Shape_mpq_class_affine_dimension:
  bd_shape_mpq_class -> int




val ppl_BD_Shape_mpq_class_relation_with_constraint:
  bd_shape_mpq_class -> linear_constraint
  -> poly_con_relation list

val ppl_BD_Shape_mpq_class_relation_with_generator:
  bd_shape_mpq_class -> linear_generator
  -> poly_gen_relation list

val ppl_BD_Shape_mpq_class_relation_with_congruence:
  bd_shape_mpq_class -> linear_congruence
  -> poly_con_relation list





val ppl_BD_Shape_mpq_class_get_constraints:
  bd_shape_mpq_class -> constraint_system

val ppl_BD_Shape_mpq_class_get_congruences:
  bd_shape_mpq_class -> congruence_system





val ppl_BD_Shape_mpq_class_get_minimized_constraints:
  bd_shape_mpq_class -> constraint_system

val ppl_BD_Shape_mpq_class_get_minimized_congruences:
  bd_shape_mpq_class -> congruence_system





val ppl_BD_Shape_mpq_class_is_empty:
  bd_shape_mpq_class -> bool

val ppl_BD_Shape_mpq_class_is_universe:
  bd_shape_mpq_class -> bool

val ppl_BD_Shape_mpq_class_is_bounded:
  bd_shape_mpq_class -> bool

val ppl_BD_Shape_mpq_class_contains_integer_point:
  bd_shape_mpq_class -> bool

val ppl_BD_Shape_mpq_class_is_topologically_closed:
  bd_shape_mpq_class -> bool

val ppl_BD_Shape_mpq_class_is_discrete:
  bd_shape_mpq_class -> bool





val ppl_BD_Shape_mpq_class_topological_closure_assign:
  bd_shape_mpq_class -> unit





val ppl_BD_Shape_mpq_class_bounds_from_above:
  bd_shape_mpq_class -> linear_expression -> bool

val ppl_BD_Shape_mpq_class_bounds_from_below:
  bd_shape_mpq_class -> linear_expression -> bool





val ppl_BD_Shape_mpq_class_maximize:
  bd_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_BD_Shape_mpq_class_minimize:
  bd_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_BD_Shape_mpq_class_maximize_with_point:
  bd_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_BD_Shape_mpq_class_minimize_with_point:
  bd_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_BD_Shape_mpq_class_frequency:
  bd_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_BD_Shape_mpq_class_contains_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool

val ppl_BD_Shape_mpq_class_strictly_contains_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool

val ppl_BD_Shape_mpq_class_is_disjoint_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool





val ppl_BD_Shape_mpq_class_equals_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool




val ppl_BD_Shape_mpq_class_OK:
  bd_shape_mpq_class -> bool



val ppl_BD_Shape_mpq_class_add_constraint:
  bd_shape_mpq_class -> linear_constraint -> unit

val ppl_BD_Shape_mpq_class_add_congruence:
  bd_shape_mpq_class -> linear_congruence -> unit





val ppl_BD_Shape_mpq_class_add_constraints:
  bd_shape_mpq_class -> constraint_system -> unit

val ppl_BD_Shape_mpq_class_add_congruences:
  bd_shape_mpq_class -> congruence_system -> unit





val ppl_BD_Shape_mpq_class_refine_with_constraint:
  bd_shape_mpq_class -> linear_constraint -> unit

val ppl_BD_Shape_mpq_class_refine_with_congruence:
  bd_shape_mpq_class -> linear_congruence -> unit





val ppl_BD_Shape_mpq_class_refine_with_constraints:
  bd_shape_mpq_class -> constraint_system -> unit

val ppl_BD_Shape_mpq_class_refine_with_congruences:
  bd_shape_mpq_class -> congruence_system -> unit





val ppl_BD_Shape_mpq_class_intersection_assign:
   bd_shape_mpq_class -> bd_shape_mpq_class -> unit

val ppl_BD_Shape_mpq_class_upper_bound_assign:
   bd_shape_mpq_class -> bd_shape_mpq_class -> unit

val ppl_BD_Shape_mpq_class_difference_assign:
   bd_shape_mpq_class -> bd_shape_mpq_class -> unit

val ppl_BD_Shape_mpq_class_concatenate_assign:
   bd_shape_mpq_class -> bd_shape_mpq_class -> unit

val ppl_BD_Shape_mpq_class_time_elapse_assign:
   bd_shape_mpq_class -> bd_shape_mpq_class -> unit





val ppl_BD_Shape_mpq_class_upper_bound_assign_if_exact:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool





val ppl_BD_Shape_mpq_class_simplify_using_context_assign:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool




val ppl_BD_Shape_mpq_class_constrains:
  bd_shape_mpq_class -> int -> bool




val ppl_BD_Shape_mpq_class_unconstrain_space_dimension:
  bd_shape_mpq_class -> int -> unit




val ppl_BD_Shape_mpq_class_unconstrain_space_dimensions:
  bd_shape_mpq_class -> int list -> unit




val ppl_BD_Shape_mpq_class_affine_image:
  bd_shape_mpq_class -> int -> linear_expression -> Z.t -> unit

val ppl_BD_Shape_mpq_class_affine_preimage:
  bd_shape_mpq_class -> int -> linear_expression -> Z.t -> unit





val ppl_BD_Shape_mpq_class_bounded_affine_image:
  bd_shape_mpq_class -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_BD_Shape_mpq_class_bounded_affine_preimage:
  bd_shape_mpq_class -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_BD_Shape_mpq_class_generalized_affine_image:
  bd_shape_mpq_class -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_BD_Shape_mpq_class_generalized_affine_preimage:
  bd_shape_mpq_class -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_BD_Shape_mpq_class_generalized_affine_image_lhs_rhs:
  bd_shape_mpq_class -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_BD_Shape_mpq_class_generalized_affine_preimage_lhs_rhs:
  bd_shape_mpq_class -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_BD_Shape_mpq_class_add_space_dimensions_and_embed:
  bd_shape_mpq_class -> int -> unit

val ppl_BD_Shape_mpq_class_add_space_dimensions_and_project:
  bd_shape_mpq_class -> int -> unit





val ppl_BD_Shape_mpq_class_remove_space_dimensions:
  bd_shape_mpq_class -> int list -> unit




val ppl_BD_Shape_mpq_class_remove_higher_space_dimensions:
  bd_shape_mpq_class -> int -> unit




val ppl_BD_Shape_mpq_class_expand_space_dimension:
  bd_shape_mpq_class -> int -> int -> unit



val ppl_BD_Shape_mpq_class_fold_space_dimensions:
  bd_shape_mpq_class -> int list -> int -> unit




val ppl_BD_Shape_mpq_class_map_space_dimensions:
  bd_shape_mpq_class -> (int*int) list -> unit



val ppl_BD_Shape_mpq_class_drop_some_non_integer_points:
  bd_shape_mpq_class -> complexity_class -> unit




val ppl_BD_Shape_mpq_class_drop_some_non_integer_points_2:
  bd_shape_mpq_class -> int list -> complexity_class -> unit




val ppl_BD_Shape_mpq_class_ascii_dump:
  bd_shape_mpq_class -> string




val ppl_BD_Shape_mpq_class_external_memory_in_bytes:
  bd_shape_mpq_class  -> int

val ppl_BD_Shape_mpq_class_total_memory_in_bytes:
  bd_shape_mpq_class  -> int





val ppl_BD_Shape_mpq_class_BHMZ05_widening_assign_with_tokens:
  bd_shape_mpq_class -> bd_shape_mpq_class -> int -> int

val ppl_BD_Shape_mpq_class_H79_widening_assign_with_tokens:
  bd_shape_mpq_class -> bd_shape_mpq_class -> int -> int





val ppl_BD_Shape_mpq_class_BHMZ05_widening_assign:
  bd_shape_mpq_class -> bd_shape_mpq_class -> unit

val ppl_BD_Shape_mpq_class_H79_widening_assign:
  bd_shape_mpq_class -> bd_shape_mpq_class -> unit





val ppl_BD_Shape_mpq_class_widening_assign_with_tokens:
  bd_shape_mpq_class -> bd_shape_mpq_class -> int -> int




val ppl_BD_Shape_mpq_class_widening_assign:
  bd_shape_mpq_class -> bd_shape_mpq_class -> unit




val ppl_BD_Shape_mpq_class_limited_BHMZ05_extrapolation_assign_with_tokens:
  bd_shape_mpq_class -> bd_shape_mpq_class
  -> constraint_system -> int -> int

val ppl_BD_Shape_mpq_class_limited_H79_extrapolation_assign_with_tokens:
  bd_shape_mpq_class -> bd_shape_mpq_class
  -> constraint_system -> int -> int

val ppl_BD_Shape_mpq_class_limited_CC76_extrapolation_assign_with_tokens:
  bd_shape_mpq_class -> bd_shape_mpq_class
  -> constraint_system -> int -> int







val ppl_BD_Shape_mpq_class_limited_BHMZ05_extrapolation_assign:
  bd_shape_mpq_class -> bd_shape_mpq_class
  -> constraint_system -> unit

val ppl_BD_Shape_mpq_class_limited_H79_extrapolation_assign:
  bd_shape_mpq_class -> bd_shape_mpq_class
  -> constraint_system -> unit

val ppl_BD_Shape_mpq_class_limited_CC76_extrapolation_assign:
  bd_shape_mpq_class -> bd_shape_mpq_class
  -> constraint_system -> unit







val ppl_BD_Shape_mpq_class_CC76_extrapolation_assign_with_tokens:
  bd_shape_mpq_class  -> bd_shape_mpq_class -> int -> int





val ppl_BD_Shape_mpq_class_CC76_extrapolation_assign:
  bd_shape_mpq_class  -> bd_shape_mpq_class -> unit





val ppl_BD_Shape_mpq_class_CC76_narrowing_assign:
  bd_shape_mpq_class  -> bd_shape_mpq_class -> unit





val ppl_BD_Shape_mpq_class_linear_partition:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bd_shape_mpq_class * pointset_powerset_nnc_polyhedron





val ppl_BD_Shape_mpq_class_wrap_assign: bd_shape_mpq_class
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bool

val ppl_termination_test_PR_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bool






val ppl_one_affine_ranking_function_MS_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_BD_Shape_mpq_class:
  bd_shape_mpq_class -> polyhedron

val ppl_all_affine_ranking_functions_PR_BD_Shape_mpq_class:
  bd_shape_mpq_class -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_BD_Shape_mpq_class:
  bd_shape_mpq_class -> polyhedron * polyhedron





val ppl_termination_test_MS_BD_Shape_mpq_class_2:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool

val ppl_termination_test_PR_BD_Shape_mpq_class_2:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool






val ppl_one_affine_ranking_function_MS_BD_Shape_mpq_class_2:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_BD_Shape_mpq_class_2:
  bd_shape_mpq_class -> bd_shape_mpq_class -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_BD_Shape_mpq_class_2:
  bd_shape_mpq_class -> bd_shape_mpq_class -> polyhedron

val ppl_all_affine_ranking_functions_PR_BD_Shape_mpq_class_2:
  bd_shape_mpq_class -> bd_shape_mpq_class -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_BD_Shape_mpq_class_2:
  bd_shape_mpq_class -> bd_shape_mpq_class -> polyhedron * polyhedron








val ppl_new_Octagonal_Shape_mpz_class_from_space_dimension:
  int -> degenerate_element -> octagonal_shape_mpz_class





val ppl_new_Octagonal_Shape_mpz_class_from_C_Polyhedron:
  polyhedron -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_NNC_Polyhedron:
  polyhedron -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Grid:
  grid -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Rational_Box:
  rational_box -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Double_Box:
  double_box -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_BD_Shape_double:
  bd_shape_double -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Octagonal_Shape_double:
  octagonal_shape_double -> octagonal_shape_mpz_class






val ppl_new_Octagonal_Shape_mpz_class_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Grid_with_complexity:
  grid -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Double_Box_with_complexity:
  double_box -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> octagonal_shape_mpz_class






val ppl_new_Octagonal_Shape_mpz_class_from_constraints:
  constraint_system -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_congruences:
  congruence_system -> octagonal_shape_mpz_class

val ppl_new_Octagonal_Shape_mpz_class_from_generators:
  generator_system -> octagonal_shape_mpz_class






val ppl_Octagonal_Shape_mpz_class_swap:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> unit




val ppl_Octagonal_Shape_mpz_class_space_dimension:
  octagonal_shape_mpz_class -> int
val ppl_Octagonal_Shape_mpz_class_affine_dimension:
  octagonal_shape_mpz_class -> int




val ppl_Octagonal_Shape_mpz_class_relation_with_constraint:
  octagonal_shape_mpz_class -> linear_constraint
  -> poly_con_relation list

val ppl_Octagonal_Shape_mpz_class_relation_with_generator:
  octagonal_shape_mpz_class -> linear_generator
  -> poly_gen_relation list

val ppl_Octagonal_Shape_mpz_class_relation_with_congruence:
  octagonal_shape_mpz_class -> linear_congruence
  -> poly_con_relation list





val ppl_Octagonal_Shape_mpz_class_get_constraints:
  octagonal_shape_mpz_class -> constraint_system

val ppl_Octagonal_Shape_mpz_class_get_congruences:
  octagonal_shape_mpz_class -> congruence_system





val ppl_Octagonal_Shape_mpz_class_get_minimized_constraints:
  octagonal_shape_mpz_class -> constraint_system

val ppl_Octagonal_Shape_mpz_class_get_minimized_congruences:
  octagonal_shape_mpz_class -> congruence_system





val ppl_Octagonal_Shape_mpz_class_is_empty:
  octagonal_shape_mpz_class -> bool

val ppl_Octagonal_Shape_mpz_class_is_universe:
  octagonal_shape_mpz_class -> bool

val ppl_Octagonal_Shape_mpz_class_is_bounded:
  octagonal_shape_mpz_class -> bool

val ppl_Octagonal_Shape_mpz_class_contains_integer_point:
  octagonal_shape_mpz_class -> bool

val ppl_Octagonal_Shape_mpz_class_is_topologically_closed:
  octagonal_shape_mpz_class -> bool

val ppl_Octagonal_Shape_mpz_class_is_discrete:
  octagonal_shape_mpz_class -> bool





val ppl_Octagonal_Shape_mpz_class_topological_closure_assign:
  octagonal_shape_mpz_class -> unit





val ppl_Octagonal_Shape_mpz_class_bounds_from_above:
  octagonal_shape_mpz_class -> linear_expression -> bool

val ppl_Octagonal_Shape_mpz_class_bounds_from_below:
  octagonal_shape_mpz_class -> linear_expression -> bool





val ppl_Octagonal_Shape_mpz_class_maximize:
  octagonal_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Octagonal_Shape_mpz_class_minimize:
  octagonal_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Octagonal_Shape_mpz_class_maximize_with_point:
  octagonal_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Octagonal_Shape_mpz_class_minimize_with_point:
  octagonal_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Octagonal_Shape_mpz_class_frequency:
  octagonal_shape_mpz_class -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_Octagonal_Shape_mpz_class_contains_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool

val ppl_Octagonal_Shape_mpz_class_strictly_contains_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool

val ppl_Octagonal_Shape_mpz_class_is_disjoint_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool





val ppl_Octagonal_Shape_mpz_class_equals_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool




val ppl_Octagonal_Shape_mpz_class_OK:
  octagonal_shape_mpz_class -> bool



val ppl_Octagonal_Shape_mpz_class_add_constraint:
  octagonal_shape_mpz_class -> linear_constraint -> unit

val ppl_Octagonal_Shape_mpz_class_add_congruence:
  octagonal_shape_mpz_class -> linear_congruence -> unit





val ppl_Octagonal_Shape_mpz_class_add_constraints:
  octagonal_shape_mpz_class -> constraint_system -> unit

val ppl_Octagonal_Shape_mpz_class_add_congruences:
  octagonal_shape_mpz_class -> congruence_system -> unit





val ppl_Octagonal_Shape_mpz_class_refine_with_constraint:
  octagonal_shape_mpz_class -> linear_constraint -> unit

val ppl_Octagonal_Shape_mpz_class_refine_with_congruence:
  octagonal_shape_mpz_class -> linear_congruence -> unit





val ppl_Octagonal_Shape_mpz_class_refine_with_constraints:
  octagonal_shape_mpz_class -> constraint_system -> unit

val ppl_Octagonal_Shape_mpz_class_refine_with_congruences:
  octagonal_shape_mpz_class -> congruence_system -> unit





val ppl_Octagonal_Shape_mpz_class_intersection_assign:
   octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> unit

val ppl_Octagonal_Shape_mpz_class_upper_bound_assign:
   octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> unit

val ppl_Octagonal_Shape_mpz_class_difference_assign:
   octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> unit

val ppl_Octagonal_Shape_mpz_class_concatenate_assign:
   octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> unit

val ppl_Octagonal_Shape_mpz_class_time_elapse_assign:
   octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> unit





val ppl_Octagonal_Shape_mpz_class_upper_bound_assign_if_exact:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool





val ppl_Octagonal_Shape_mpz_class_simplify_using_context_assign:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool




val ppl_Octagonal_Shape_mpz_class_constrains:
  octagonal_shape_mpz_class -> int -> bool




val ppl_Octagonal_Shape_mpz_class_unconstrain_space_dimension:
  octagonal_shape_mpz_class -> int -> unit




val ppl_Octagonal_Shape_mpz_class_unconstrain_space_dimensions:
  octagonal_shape_mpz_class -> int list -> unit




val ppl_Octagonal_Shape_mpz_class_affine_image:
  octagonal_shape_mpz_class -> int -> linear_expression -> Z.t -> unit

val ppl_Octagonal_Shape_mpz_class_affine_preimage:
  octagonal_shape_mpz_class -> int -> linear_expression -> Z.t -> unit





val ppl_Octagonal_Shape_mpz_class_bounded_affine_image:
  octagonal_shape_mpz_class -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Octagonal_Shape_mpz_class_bounded_affine_preimage:
  octagonal_shape_mpz_class -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Octagonal_Shape_mpz_class_generalized_affine_image:
  octagonal_shape_mpz_class -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Octagonal_Shape_mpz_class_generalized_affine_preimage:
  octagonal_shape_mpz_class -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Octagonal_Shape_mpz_class_generalized_affine_image_lhs_rhs:
  octagonal_shape_mpz_class -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Octagonal_Shape_mpz_class_generalized_affine_preimage_lhs_rhs:
  octagonal_shape_mpz_class -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Octagonal_Shape_mpz_class_add_space_dimensions_and_embed:
  octagonal_shape_mpz_class -> int -> unit

val ppl_Octagonal_Shape_mpz_class_add_space_dimensions_and_project:
  octagonal_shape_mpz_class -> int -> unit





val ppl_Octagonal_Shape_mpz_class_remove_space_dimensions:
  octagonal_shape_mpz_class -> int list -> unit




val ppl_Octagonal_Shape_mpz_class_remove_higher_space_dimensions:
  octagonal_shape_mpz_class -> int -> unit




val ppl_Octagonal_Shape_mpz_class_expand_space_dimension:
  octagonal_shape_mpz_class -> int -> int -> unit



val ppl_Octagonal_Shape_mpz_class_fold_space_dimensions:
  octagonal_shape_mpz_class -> int list -> int -> unit




val ppl_Octagonal_Shape_mpz_class_map_space_dimensions:
  octagonal_shape_mpz_class -> (int*int) list -> unit



val ppl_Octagonal_Shape_mpz_class_drop_some_non_integer_points:
  octagonal_shape_mpz_class -> complexity_class -> unit




val ppl_Octagonal_Shape_mpz_class_drop_some_non_integer_points_2:
  octagonal_shape_mpz_class -> int list -> complexity_class -> unit




val ppl_Octagonal_Shape_mpz_class_ascii_dump:
  octagonal_shape_mpz_class -> string




val ppl_Octagonal_Shape_mpz_class_external_memory_in_bytes:
  octagonal_shape_mpz_class  -> int

val ppl_Octagonal_Shape_mpz_class_total_memory_in_bytes:
  octagonal_shape_mpz_class  -> int





val ppl_Octagonal_Shape_mpz_class_BHMZ05_widening_assign_with_tokens:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> int -> int





val ppl_Octagonal_Shape_mpz_class_BHMZ05_widening_assign:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> unit





val ppl_Octagonal_Shape_mpz_class_widening_assign_with_tokens:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> int -> int




val ppl_Octagonal_Shape_mpz_class_widening_assign:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> unit




val ppl_Octagonal_Shape_mpz_class_limited_BHMZ05_extrapolation_assign_with_tokens:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class
  -> constraint_system -> int -> int

val ppl_Octagonal_Shape_mpz_class_limited_CC76_extrapolation_assign_with_tokens:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class
  -> constraint_system -> int -> int







val ppl_Octagonal_Shape_mpz_class_limited_BHMZ05_extrapolation_assign:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class
  -> constraint_system -> unit

val ppl_Octagonal_Shape_mpz_class_limited_CC76_extrapolation_assign:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class
  -> constraint_system -> unit







val ppl_Octagonal_Shape_mpz_class_CC76_extrapolation_assign_with_tokens:
  octagonal_shape_mpz_class  -> octagonal_shape_mpz_class -> int -> int





val ppl_Octagonal_Shape_mpz_class_CC76_extrapolation_assign:
  octagonal_shape_mpz_class  -> octagonal_shape_mpz_class -> unit





val ppl_Octagonal_Shape_mpz_class_CC76_narrowing_assign:
  octagonal_shape_mpz_class  -> octagonal_shape_mpz_class -> unit





val ppl_Octagonal_Shape_mpz_class_linear_partition:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> octagonal_shape_mpz_class * pointset_powerset_nnc_polyhedron





val ppl_Octagonal_Shape_mpz_class_wrap_assign: octagonal_shape_mpz_class
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> bool

val ppl_termination_test_PR_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> bool






val ppl_one_affine_ranking_function_MS_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> polyhedron

val ppl_all_affine_ranking_functions_PR_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> polyhedron * polyhedron





val ppl_termination_test_MS_Octagonal_Shape_mpz_class_2:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool

val ppl_termination_test_PR_Octagonal_Shape_mpz_class_2:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool






val ppl_one_affine_ranking_function_MS_Octagonal_Shape_mpz_class_2:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Octagonal_Shape_mpz_class_2:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Octagonal_Shape_mpz_class_2:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> polyhedron

val ppl_all_affine_ranking_functions_PR_Octagonal_Shape_mpz_class_2:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Octagonal_Shape_mpz_class_2:
  octagonal_shape_mpz_class -> octagonal_shape_mpz_class -> polyhedron * polyhedron








val ppl_new_Octagonal_Shape_mpq_class_from_space_dimension:
  int -> degenerate_element -> octagonal_shape_mpq_class





val ppl_new_Octagonal_Shape_mpq_class_from_C_Polyhedron:
  polyhedron -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_NNC_Polyhedron:
  polyhedron -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Grid:
  grid -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Rational_Box:
  rational_box -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Double_Box:
  double_box -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_BD_Shape_double:
  bd_shape_double -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Octagonal_Shape_double:
  octagonal_shape_double -> octagonal_shape_mpq_class






val ppl_new_Octagonal_Shape_mpq_class_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Grid_with_complexity:
  grid -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Double_Box_with_complexity:
  double_box -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> octagonal_shape_mpq_class






val ppl_new_Octagonal_Shape_mpq_class_from_constraints:
  constraint_system -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_congruences:
  congruence_system -> octagonal_shape_mpq_class

val ppl_new_Octagonal_Shape_mpq_class_from_generators:
  generator_system -> octagonal_shape_mpq_class






val ppl_Octagonal_Shape_mpq_class_swap:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> unit




val ppl_Octagonal_Shape_mpq_class_space_dimension:
  octagonal_shape_mpq_class -> int
val ppl_Octagonal_Shape_mpq_class_affine_dimension:
  octagonal_shape_mpq_class -> int




val ppl_Octagonal_Shape_mpq_class_relation_with_constraint:
  octagonal_shape_mpq_class -> linear_constraint
  -> poly_con_relation list

val ppl_Octagonal_Shape_mpq_class_relation_with_generator:
  octagonal_shape_mpq_class -> linear_generator
  -> poly_gen_relation list

val ppl_Octagonal_Shape_mpq_class_relation_with_congruence:
  octagonal_shape_mpq_class -> linear_congruence
  -> poly_con_relation list





val ppl_Octagonal_Shape_mpq_class_get_constraints:
  octagonal_shape_mpq_class -> constraint_system

val ppl_Octagonal_Shape_mpq_class_get_congruences:
  octagonal_shape_mpq_class -> congruence_system





val ppl_Octagonal_Shape_mpq_class_get_minimized_constraints:
  octagonal_shape_mpq_class -> constraint_system

val ppl_Octagonal_Shape_mpq_class_get_minimized_congruences:
  octagonal_shape_mpq_class -> congruence_system





val ppl_Octagonal_Shape_mpq_class_is_empty:
  octagonal_shape_mpq_class -> bool

val ppl_Octagonal_Shape_mpq_class_is_universe:
  octagonal_shape_mpq_class -> bool

val ppl_Octagonal_Shape_mpq_class_is_bounded:
  octagonal_shape_mpq_class -> bool

val ppl_Octagonal_Shape_mpq_class_contains_integer_point:
  octagonal_shape_mpq_class -> bool

val ppl_Octagonal_Shape_mpq_class_is_topologically_closed:
  octagonal_shape_mpq_class -> bool

val ppl_Octagonal_Shape_mpq_class_is_discrete:
  octagonal_shape_mpq_class -> bool





val ppl_Octagonal_Shape_mpq_class_topological_closure_assign:
  octagonal_shape_mpq_class -> unit





val ppl_Octagonal_Shape_mpq_class_bounds_from_above:
  octagonal_shape_mpq_class -> linear_expression -> bool

val ppl_Octagonal_Shape_mpq_class_bounds_from_below:
  octagonal_shape_mpq_class -> linear_expression -> bool





val ppl_Octagonal_Shape_mpq_class_maximize:
  octagonal_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Octagonal_Shape_mpq_class_minimize:
  octagonal_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Octagonal_Shape_mpq_class_maximize_with_point:
  octagonal_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Octagonal_Shape_mpq_class_minimize_with_point:
  octagonal_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Octagonal_Shape_mpq_class_frequency:
  octagonal_shape_mpq_class -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_Octagonal_Shape_mpq_class_contains_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool

val ppl_Octagonal_Shape_mpq_class_strictly_contains_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool

val ppl_Octagonal_Shape_mpq_class_is_disjoint_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool





val ppl_Octagonal_Shape_mpq_class_equals_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool




val ppl_Octagonal_Shape_mpq_class_OK:
  octagonal_shape_mpq_class -> bool



val ppl_Octagonal_Shape_mpq_class_add_constraint:
  octagonal_shape_mpq_class -> linear_constraint -> unit

val ppl_Octagonal_Shape_mpq_class_add_congruence:
  octagonal_shape_mpq_class -> linear_congruence -> unit





val ppl_Octagonal_Shape_mpq_class_add_constraints:
  octagonal_shape_mpq_class -> constraint_system -> unit

val ppl_Octagonal_Shape_mpq_class_add_congruences:
  octagonal_shape_mpq_class -> congruence_system -> unit





val ppl_Octagonal_Shape_mpq_class_refine_with_constraint:
  octagonal_shape_mpq_class -> linear_constraint -> unit

val ppl_Octagonal_Shape_mpq_class_refine_with_congruence:
  octagonal_shape_mpq_class -> linear_congruence -> unit





val ppl_Octagonal_Shape_mpq_class_refine_with_constraints:
  octagonal_shape_mpq_class -> constraint_system -> unit

val ppl_Octagonal_Shape_mpq_class_refine_with_congruences:
  octagonal_shape_mpq_class -> congruence_system -> unit





val ppl_Octagonal_Shape_mpq_class_intersection_assign:
   octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> unit

val ppl_Octagonal_Shape_mpq_class_upper_bound_assign:
   octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> unit

val ppl_Octagonal_Shape_mpq_class_difference_assign:
   octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> unit

val ppl_Octagonal_Shape_mpq_class_concatenate_assign:
   octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> unit

val ppl_Octagonal_Shape_mpq_class_time_elapse_assign:
   octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> unit





val ppl_Octagonal_Shape_mpq_class_upper_bound_assign_if_exact:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool





val ppl_Octagonal_Shape_mpq_class_simplify_using_context_assign:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool




val ppl_Octagonal_Shape_mpq_class_constrains:
  octagonal_shape_mpq_class -> int -> bool




val ppl_Octagonal_Shape_mpq_class_unconstrain_space_dimension:
  octagonal_shape_mpq_class -> int -> unit




val ppl_Octagonal_Shape_mpq_class_unconstrain_space_dimensions:
  octagonal_shape_mpq_class -> int list -> unit




val ppl_Octagonal_Shape_mpq_class_affine_image:
  octagonal_shape_mpq_class -> int -> linear_expression -> Z.t -> unit

val ppl_Octagonal_Shape_mpq_class_affine_preimage:
  octagonal_shape_mpq_class -> int -> linear_expression -> Z.t -> unit





val ppl_Octagonal_Shape_mpq_class_bounded_affine_image:
  octagonal_shape_mpq_class -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Octagonal_Shape_mpq_class_bounded_affine_preimage:
  octagonal_shape_mpq_class -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Octagonal_Shape_mpq_class_generalized_affine_image:
  octagonal_shape_mpq_class -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Octagonal_Shape_mpq_class_generalized_affine_preimage:
  octagonal_shape_mpq_class -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Octagonal_Shape_mpq_class_generalized_affine_image_lhs_rhs:
  octagonal_shape_mpq_class -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Octagonal_Shape_mpq_class_generalized_affine_preimage_lhs_rhs:
  octagonal_shape_mpq_class -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Octagonal_Shape_mpq_class_add_space_dimensions_and_embed:
  octagonal_shape_mpq_class -> int -> unit

val ppl_Octagonal_Shape_mpq_class_add_space_dimensions_and_project:
  octagonal_shape_mpq_class -> int -> unit





val ppl_Octagonal_Shape_mpq_class_remove_space_dimensions:
  octagonal_shape_mpq_class -> int list -> unit




val ppl_Octagonal_Shape_mpq_class_remove_higher_space_dimensions:
  octagonal_shape_mpq_class -> int -> unit




val ppl_Octagonal_Shape_mpq_class_expand_space_dimension:
  octagonal_shape_mpq_class -> int -> int -> unit



val ppl_Octagonal_Shape_mpq_class_fold_space_dimensions:
  octagonal_shape_mpq_class -> int list -> int -> unit




val ppl_Octagonal_Shape_mpq_class_map_space_dimensions:
  octagonal_shape_mpq_class -> (int*int) list -> unit



val ppl_Octagonal_Shape_mpq_class_drop_some_non_integer_points:
  octagonal_shape_mpq_class -> complexity_class -> unit




val ppl_Octagonal_Shape_mpq_class_drop_some_non_integer_points_2:
  octagonal_shape_mpq_class -> int list -> complexity_class -> unit




val ppl_Octagonal_Shape_mpq_class_ascii_dump:
  octagonal_shape_mpq_class -> string




val ppl_Octagonal_Shape_mpq_class_external_memory_in_bytes:
  octagonal_shape_mpq_class  -> int

val ppl_Octagonal_Shape_mpq_class_total_memory_in_bytes:
  octagonal_shape_mpq_class  -> int





val ppl_Octagonal_Shape_mpq_class_BHMZ05_widening_assign_with_tokens:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> int -> int





val ppl_Octagonal_Shape_mpq_class_BHMZ05_widening_assign:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> unit





val ppl_Octagonal_Shape_mpq_class_widening_assign_with_tokens:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> int -> int




val ppl_Octagonal_Shape_mpq_class_widening_assign:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> unit




val ppl_Octagonal_Shape_mpq_class_limited_BHMZ05_extrapolation_assign_with_tokens:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class
  -> constraint_system -> int -> int

val ppl_Octagonal_Shape_mpq_class_limited_CC76_extrapolation_assign_with_tokens:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class
  -> constraint_system -> int -> int







val ppl_Octagonal_Shape_mpq_class_limited_BHMZ05_extrapolation_assign:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class
  -> constraint_system -> unit

val ppl_Octagonal_Shape_mpq_class_limited_CC76_extrapolation_assign:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class
  -> constraint_system -> unit







val ppl_Octagonal_Shape_mpq_class_CC76_extrapolation_assign_with_tokens:
  octagonal_shape_mpq_class  -> octagonal_shape_mpq_class -> int -> int





val ppl_Octagonal_Shape_mpq_class_CC76_extrapolation_assign:
  octagonal_shape_mpq_class  -> octagonal_shape_mpq_class -> unit





val ppl_Octagonal_Shape_mpq_class_CC76_narrowing_assign:
  octagonal_shape_mpq_class  -> octagonal_shape_mpq_class -> unit





val ppl_Octagonal_Shape_mpq_class_linear_partition:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> octagonal_shape_mpq_class * pointset_powerset_nnc_polyhedron





val ppl_Octagonal_Shape_mpq_class_wrap_assign: octagonal_shape_mpq_class
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> bool

val ppl_termination_test_PR_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> bool






val ppl_one_affine_ranking_function_MS_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> polyhedron

val ppl_all_affine_ranking_functions_PR_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> polyhedron * polyhedron





val ppl_termination_test_MS_Octagonal_Shape_mpq_class_2:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool

val ppl_termination_test_PR_Octagonal_Shape_mpq_class_2:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool






val ppl_one_affine_ranking_function_MS_Octagonal_Shape_mpq_class_2:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Octagonal_Shape_mpq_class_2:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Octagonal_Shape_mpq_class_2:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> polyhedron

val ppl_all_affine_ranking_functions_PR_Octagonal_Shape_mpq_class_2:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Octagonal_Shape_mpq_class_2:
  octagonal_shape_mpq_class -> octagonal_shape_mpq_class -> polyhedron * polyhedron








val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_space_dimension:
  int -> degenerate_element -> constraints_product_c_polyhedron_grid





val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_C_Polyhedron:
  polyhedron -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_NNC_Polyhedron:
  polyhedron -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Grid:
  grid -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Rational_Box:
  rational_box -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Double_Box:
  double_box -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_BD_Shape_double:
  bd_shape_double -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Octagonal_Shape_double:
  octagonal_shape_double -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Constraints_Product_C_Polyhedron_Grid:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid






val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Grid_with_complexity:
  grid -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Double_Box_with_complexity:
  double_box -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_Constraints_Product_C_Polyhedron_Grid_with_complexity:
  constraints_product_c_polyhedron_grid -> complexity_class -> constraints_product_c_polyhedron_grid






val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_constraints:
  constraint_system -> constraints_product_c_polyhedron_grid

val ppl_new_Constraints_Product_C_Polyhedron_Grid_from_congruences:
  congruence_system -> constraints_product_c_polyhedron_grid






val ppl_Constraints_Product_C_Polyhedron_Grid_swap:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_space_dimension:
  constraints_product_c_polyhedron_grid -> int
val ppl_Constraints_Product_C_Polyhedron_Grid_affine_dimension:
  constraints_product_c_polyhedron_grid -> int




val ppl_Constraints_Product_C_Polyhedron_Grid_relation_with_constraint:
  constraints_product_c_polyhedron_grid -> linear_constraint
  -> poly_con_relation list

val ppl_Constraints_Product_C_Polyhedron_Grid_relation_with_generator:
  constraints_product_c_polyhedron_grid -> linear_generator
  -> poly_gen_relation list

val ppl_Constraints_Product_C_Polyhedron_Grid_relation_with_congruence:
  constraints_product_c_polyhedron_grid -> linear_congruence
  -> poly_con_relation list





val ppl_Constraints_Product_C_Polyhedron_Grid_is_empty:
  constraints_product_c_polyhedron_grid -> bool

val ppl_Constraints_Product_C_Polyhedron_Grid_is_universe:
  constraints_product_c_polyhedron_grid -> bool

val ppl_Constraints_Product_C_Polyhedron_Grid_is_bounded:
  constraints_product_c_polyhedron_grid -> bool

val ppl_Constraints_Product_C_Polyhedron_Grid_is_topologically_closed:
  constraints_product_c_polyhedron_grid -> bool

val ppl_Constraints_Product_C_Polyhedron_Grid_is_discrete:
  constraints_product_c_polyhedron_grid -> bool





val ppl_Constraints_Product_C_Polyhedron_Grid_topological_closure_assign:
  constraints_product_c_polyhedron_grid -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_bounds_from_above:
  constraints_product_c_polyhedron_grid -> linear_expression -> bool

val ppl_Constraints_Product_C_Polyhedron_Grid_bounds_from_below:
  constraints_product_c_polyhedron_grid -> linear_expression -> bool





val ppl_Constraints_Product_C_Polyhedron_Grid_maximize:
  constraints_product_c_polyhedron_grid -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Constraints_Product_C_Polyhedron_Grid_minimize:
  constraints_product_c_polyhedron_grid -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Constraints_Product_C_Polyhedron_Grid_maximize_with_point:
  constraints_product_c_polyhedron_grid -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Constraints_Product_C_Polyhedron_Grid_minimize_with_point:
  constraints_product_c_polyhedron_grid -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Constraints_Product_C_Polyhedron_Grid_contains_Constraints_Product_C_Polyhedron_Grid:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> bool

val ppl_Constraints_Product_C_Polyhedron_Grid_strictly_contains_Constraints_Product_C_Polyhedron_Grid:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> bool

val ppl_Constraints_Product_C_Polyhedron_Grid_is_disjoint_from_Constraints_Product_C_Polyhedron_Grid:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> bool





val ppl_Constraints_Product_C_Polyhedron_Grid_equals_Constraints_Product_C_Polyhedron_Grid:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> bool




val ppl_Constraints_Product_C_Polyhedron_Grid_OK:
  constraints_product_c_polyhedron_grid -> bool



val ppl_Constraints_Product_C_Polyhedron_Grid_add_constraint:
  constraints_product_c_polyhedron_grid -> linear_constraint -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_add_congruence:
  constraints_product_c_polyhedron_grid -> linear_congruence -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_add_constraints:
  constraints_product_c_polyhedron_grid -> constraint_system -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_add_congruences:
  constraints_product_c_polyhedron_grid -> congruence_system -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_refine_with_constraint:
  constraints_product_c_polyhedron_grid -> linear_constraint -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_refine_with_congruence:
  constraints_product_c_polyhedron_grid -> linear_congruence -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_refine_with_constraints:
  constraints_product_c_polyhedron_grid -> constraint_system -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_refine_with_congruences:
  constraints_product_c_polyhedron_grid -> congruence_system -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_intersection_assign:
   constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_upper_bound_assign:
   constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_difference_assign:
   constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_concatenate_assign:
   constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_time_elapse_assign:
   constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_upper_bound_assign_if_exact:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> bool





val ppl_Constraints_Product_C_Polyhedron_Grid_constrains:
  constraints_product_c_polyhedron_grid -> int -> bool




val ppl_Constraints_Product_C_Polyhedron_Grid_unconstrain_space_dimension:
  constraints_product_c_polyhedron_grid -> int -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_unconstrain_space_dimensions:
  constraints_product_c_polyhedron_grid -> int list -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_affine_image:
  constraints_product_c_polyhedron_grid -> int -> linear_expression -> Z.t -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_affine_preimage:
  constraints_product_c_polyhedron_grid -> int -> linear_expression -> Z.t -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_bounded_affine_image:
  constraints_product_c_polyhedron_grid -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Constraints_Product_C_Polyhedron_Grid_bounded_affine_preimage:
  constraints_product_c_polyhedron_grid -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_generalized_affine_image:
  constraints_product_c_polyhedron_grid -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_generalized_affine_preimage:
  constraints_product_c_polyhedron_grid -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_generalized_affine_image_lhs_rhs:
  constraints_product_c_polyhedron_grid -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_generalized_affine_preimage_lhs_rhs:
  constraints_product_c_polyhedron_grid -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_add_space_dimensions_and_embed:
  constraints_product_c_polyhedron_grid -> int -> unit

val ppl_Constraints_Product_C_Polyhedron_Grid_add_space_dimensions_and_project:
  constraints_product_c_polyhedron_grid -> int -> unit





val ppl_Constraints_Product_C_Polyhedron_Grid_remove_space_dimensions:
  constraints_product_c_polyhedron_grid -> int list -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_remove_higher_space_dimensions:
  constraints_product_c_polyhedron_grid -> int -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_expand_space_dimension:
  constraints_product_c_polyhedron_grid -> int -> int -> unit



val ppl_Constraints_Product_C_Polyhedron_Grid_fold_space_dimensions:
  constraints_product_c_polyhedron_grid -> int list -> int -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_map_space_dimensions:
  constraints_product_c_polyhedron_grid -> (int*int) list -> unit



val ppl_Constraints_Product_C_Polyhedron_Grid_drop_some_non_integer_points:
  constraints_product_c_polyhedron_grid -> complexity_class -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_drop_some_non_integer_points_2:
  constraints_product_c_polyhedron_grid -> int list -> complexity_class -> unit




val ppl_Constraints_Product_C_Polyhedron_Grid_ascii_dump:
  constraints_product_c_polyhedron_grid -> string




val ppl_Constraints_Product_C_Polyhedron_Grid_external_memory_in_bytes:
  constraints_product_c_polyhedron_grid  -> int

val ppl_Constraints_Product_C_Polyhedron_Grid_total_memory_in_bytes:
  constraints_product_c_polyhedron_grid  -> int





val ppl_Constraints_Product_C_Polyhedron_Grid_widening_assign_with_tokens:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> int -> int




val ppl_Constraints_Product_C_Polyhedron_Grid_widening_assign:
  constraints_product_c_polyhedron_grid -> constraints_product_c_polyhedron_grid -> unit









val ppl_new_Pointset_Powerset_C_Polyhedron_from_space_dimension:
  int -> degenerate_element -> pointset_powerset_c_polyhedron





val ppl_new_Pointset_Powerset_C_Polyhedron_from_Pointset_Powerset_C_Polyhedron:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron

val ppl_new_Pointset_Powerset_C_Polyhedron_from_C_Polyhedron:
  polyhedron -> pointset_powerset_c_polyhedron






val ppl_new_Pointset_Powerset_C_Polyhedron_from_Pointset_Powerset_C_Polyhedron_with_complexity:
  pointset_powerset_c_polyhedron -> complexity_class -> pointset_powerset_c_polyhedron

val ppl_new_Pointset_Powerset_C_Polyhedron_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> pointset_powerset_c_polyhedron






val ppl_new_Pointset_Powerset_C_Polyhedron_from_constraints:
  constraint_system -> pointset_powerset_c_polyhedron

val ppl_new_Pointset_Powerset_C_Polyhedron_from_congruences:
  congruence_system -> pointset_powerset_c_polyhedron






val ppl_Pointset_Powerset_C_Polyhedron_swap:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> unit




val ppl_Pointset_Powerset_C_Polyhedron_space_dimension:
  pointset_powerset_c_polyhedron -> int
val ppl_Pointset_Powerset_C_Polyhedron_affine_dimension:
  pointset_powerset_c_polyhedron -> int




val ppl_Pointset_Powerset_C_Polyhedron_relation_with_constraint:
  pointset_powerset_c_polyhedron -> linear_constraint
  -> poly_con_relation list

val ppl_Pointset_Powerset_C_Polyhedron_relation_with_generator:
  pointset_powerset_c_polyhedron -> linear_generator
  -> poly_gen_relation list

val ppl_Pointset_Powerset_C_Polyhedron_relation_with_congruence:
  pointset_powerset_c_polyhedron -> linear_congruence
  -> poly_con_relation list





val ppl_Pointset_Powerset_C_Polyhedron_is_empty:
  pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_is_universe:
  pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_is_bounded:
  pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_contains_integer_point:
  pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_is_topologically_closed:
  pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_is_discrete:
  pointset_powerset_c_polyhedron -> bool





val ppl_Pointset_Powerset_C_Polyhedron_topological_closure_assign:
  pointset_powerset_c_polyhedron -> unit

val ppl_Pointset_Powerset_C_Polyhedron_pairwise_reduce:
  pointset_powerset_c_polyhedron -> unit

val ppl_Pointset_Powerset_C_Polyhedron_omega_reduce:
  pointset_powerset_c_polyhedron -> unit





val ppl_Pointset_Powerset_C_Polyhedron_bounds_from_above:
  pointset_powerset_c_polyhedron -> linear_expression -> bool

val ppl_Pointset_Powerset_C_Polyhedron_bounds_from_below:
  pointset_powerset_c_polyhedron -> linear_expression -> bool





val ppl_Pointset_Powerset_C_Polyhedron_maximize:
  pointset_powerset_c_polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Pointset_Powerset_C_Polyhedron_minimize:
  pointset_powerset_c_polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Pointset_Powerset_C_Polyhedron_maximize_with_point:
  pointset_powerset_c_polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Pointset_Powerset_C_Polyhedron_minimize_with_point:
  pointset_powerset_c_polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Pointset_Powerset_C_Polyhedron_contains_Pointset_Powerset_C_Polyhedron:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_strictly_contains_Pointset_Powerset_C_Polyhedron:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_is_disjoint_from_Pointset_Powerset_C_Polyhedron:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_geometrically_covers_Pointset_Powerset_C_Polyhedron:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> bool

val ppl_Pointset_Powerset_C_Polyhedron_geometrically_equals_Pointset_Powerset_C_Polyhedron:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> bool





val ppl_Pointset_Powerset_C_Polyhedron_equals_Pointset_Powerset_C_Polyhedron:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> bool




val ppl_Pointset_Powerset_C_Polyhedron_OK:
  pointset_powerset_c_polyhedron -> bool



val ppl_Pointset_Powerset_C_Polyhedron_add_constraint:
  pointset_powerset_c_polyhedron -> linear_constraint -> unit

val ppl_Pointset_Powerset_C_Polyhedron_add_congruence:
  pointset_powerset_c_polyhedron -> linear_congruence -> unit





val ppl_Pointset_Powerset_C_Polyhedron_add_constraints:
  pointset_powerset_c_polyhedron -> constraint_system -> unit

val ppl_Pointset_Powerset_C_Polyhedron_add_congruences:
  pointset_powerset_c_polyhedron -> congruence_system -> unit





val ppl_Pointset_Powerset_C_Polyhedron_refine_with_constraint:
  pointset_powerset_c_polyhedron -> linear_constraint -> unit

val ppl_Pointset_Powerset_C_Polyhedron_refine_with_congruence:
  pointset_powerset_c_polyhedron -> linear_congruence -> unit





val ppl_Pointset_Powerset_C_Polyhedron_refine_with_constraints:
  pointset_powerset_c_polyhedron -> constraint_system -> unit

val ppl_Pointset_Powerset_C_Polyhedron_refine_with_congruences:
  pointset_powerset_c_polyhedron -> congruence_system -> unit





val ppl_Pointset_Powerset_C_Polyhedron_intersection_assign:
   pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> unit

val ppl_Pointset_Powerset_C_Polyhedron_upper_bound_assign:
   pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> unit

val ppl_Pointset_Powerset_C_Polyhedron_difference_assign:
   pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> unit

val ppl_Pointset_Powerset_C_Polyhedron_concatenate_assign:
   pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> unit

val ppl_Pointset_Powerset_C_Polyhedron_time_elapse_assign:
   pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> unit





val ppl_Pointset_Powerset_C_Polyhedron_upper_bound_assign_if_exact:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> bool





val ppl_Pointset_Powerset_C_Polyhedron_simplify_using_context_assign:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> bool




val ppl_Pointset_Powerset_C_Polyhedron_constrains:
  pointset_powerset_c_polyhedron -> int -> bool




val ppl_Pointset_Powerset_C_Polyhedron_unconstrain_space_dimension:
  pointset_powerset_c_polyhedron -> int -> unit




val ppl_Pointset_Powerset_C_Polyhedron_unconstrain_space_dimensions:
  pointset_powerset_c_polyhedron -> int list -> unit




val ppl_Pointset_Powerset_C_Polyhedron_affine_image:
  pointset_powerset_c_polyhedron -> int -> linear_expression -> Z.t -> unit

val ppl_Pointset_Powerset_C_Polyhedron_affine_preimage:
  pointset_powerset_c_polyhedron -> int -> linear_expression -> Z.t -> unit





val ppl_Pointset_Powerset_C_Polyhedron_bounded_affine_image:
  pointset_powerset_c_polyhedron -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Pointset_Powerset_C_Polyhedron_bounded_affine_preimage:
  pointset_powerset_c_polyhedron -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Pointset_Powerset_C_Polyhedron_generalized_affine_image:
  pointset_powerset_c_polyhedron -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Pointset_Powerset_C_Polyhedron_generalized_affine_preimage:
  pointset_powerset_c_polyhedron -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Pointset_Powerset_C_Polyhedron_generalized_affine_image_lhs_rhs:
  pointset_powerset_c_polyhedron -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Pointset_Powerset_C_Polyhedron_generalized_affine_preimage_lhs_rhs:
  pointset_powerset_c_polyhedron -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Pointset_Powerset_C_Polyhedron_add_space_dimensions_and_embed:
  pointset_powerset_c_polyhedron -> int -> unit

val ppl_Pointset_Powerset_C_Polyhedron_add_space_dimensions_and_project:
  pointset_powerset_c_polyhedron -> int -> unit





val ppl_Pointset_Powerset_C_Polyhedron_remove_space_dimensions:
  pointset_powerset_c_polyhedron -> int list -> unit




val ppl_Pointset_Powerset_C_Polyhedron_remove_higher_space_dimensions:
  pointset_powerset_c_polyhedron -> int -> unit




val ppl_Pointset_Powerset_C_Polyhedron_expand_space_dimension:
  pointset_powerset_c_polyhedron -> int -> int -> unit



val ppl_Pointset_Powerset_C_Polyhedron_fold_space_dimensions:
  pointset_powerset_c_polyhedron -> int list -> int -> unit




val ppl_Pointset_Powerset_C_Polyhedron_map_space_dimensions:
  pointset_powerset_c_polyhedron -> (int*int) list -> unit



val ppl_Pointset_Powerset_C_Polyhedron_drop_some_non_integer_points:
  pointset_powerset_c_polyhedron -> complexity_class -> unit




val ppl_Pointset_Powerset_C_Polyhedron_drop_some_non_integer_points_2:
  pointset_powerset_c_polyhedron -> int list -> complexity_class -> unit




val ppl_Pointset_Powerset_C_Polyhedron_ascii_dump:
  pointset_powerset_c_polyhedron -> string




val ppl_Pointset_Powerset_C_Polyhedron_external_memory_in_bytes:
  pointset_powerset_c_polyhedron  -> int

val ppl_Pointset_Powerset_C_Polyhedron_total_memory_in_bytes:
  pointset_powerset_c_polyhedron  -> int

val ppl_Pointset_Powerset_C_Polyhedron_size:
  pointset_powerset_c_polyhedron  -> int





type pointset_powerset_c_polyhedron_iterator

val ppl_new_Pointset_Powerset_C_Polyhedron_iterator_from_iterator:
  pointset_powerset_c_polyhedron_iterator ->  pointset_powerset_c_polyhedron_iterator




val ppl_Pointset_Powerset_C_Polyhedron_begin_iterator:
  pointset_powerset_c_polyhedron  -> pointset_powerset_c_polyhedron_iterator

val ppl_Pointset_Powerset_C_Polyhedron_end_iterator:
  pointset_powerset_c_polyhedron  -> pointset_powerset_c_polyhedron_iterator





val ppl_Pointset_Powerset_C_Polyhedron_iterator_equals_iterator:
  pointset_powerset_c_polyhedron_iterator ->  pointset_powerset_c_polyhedron_iterator -> bool




val ppl_Pointset_Powerset_C_Polyhedron_increment_iterator:
  pointset_powerset_c_polyhedron_iterator -> unit

val ppl_Pointset_Powerset_C_Polyhedron_decrement_iterator:
  pointset_powerset_c_polyhedron_iterator -> unit





val ppl_Pointset_Powerset_C_Polyhedron_get_disjunct:
  pointset_powerset_c_polyhedron_iterator  ->  polyhedron







val ppl_Pointset_Powerset_C_Polyhedron_add_disjunct:
  pointset_powerset_c_polyhedron  ->  polyhedron -> unit





val ppl_Pointset_Powerset_C_Polyhedron_drop_disjunct:
  pointset_powerset_c_polyhedron  ->  pointset_powerset_c_polyhedron_iterator -> unit




val ppl_Pointset_Powerset_C_Polyhedron_drop_disjuncts:
  pointset_powerset_c_polyhedron  ->  pointset_powerset_c_polyhedron_iterator -> pointset_powerset_c_polyhedron_iterator -> unit




val ppl_Pointset_Powerset_C_Polyhedron_BHZ03_BHRZ03_BHRZ03_widening_assign:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> unit

val ppl_Pointset_Powerset_C_Polyhedron_BHZ03_H79_H79_widening_assign:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> unit





val ppl_Pointset_Powerset_C_Polyhedron_BGP99_BHRZ03_extrapolation_assign:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> int -> unit

val ppl_Pointset_Powerset_C_Polyhedron_BGP99_H79_extrapolation_assign:
  pointset_powerset_c_polyhedron -> pointset_powerset_c_polyhedron -> int -> unit










val ppl_new_Pointset_Powerset_NNC_Polyhedron_from_space_dimension:
  int -> degenerate_element -> pointset_powerset_nnc_polyhedron





val ppl_new_Pointset_Powerset_NNC_Polyhedron_from_Pointset_Powerset_NNC_Polyhedron:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron

val ppl_new_Pointset_Powerset_NNC_Polyhedron_from_NNC_Polyhedron:
  polyhedron -> pointset_powerset_nnc_polyhedron






val ppl_new_Pointset_Powerset_NNC_Polyhedron_from_Pointset_Powerset_NNC_Polyhedron_with_complexity:
  pointset_powerset_nnc_polyhedron -> complexity_class -> pointset_powerset_nnc_polyhedron

val ppl_new_Pointset_Powerset_NNC_Polyhedron_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> pointset_powerset_nnc_polyhedron






val ppl_new_Pointset_Powerset_NNC_Polyhedron_from_constraints:
  constraint_system -> pointset_powerset_nnc_polyhedron

val ppl_new_Pointset_Powerset_NNC_Polyhedron_from_congruences:
  congruence_system -> pointset_powerset_nnc_polyhedron






val ppl_Pointset_Powerset_NNC_Polyhedron_swap:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_space_dimension:
  pointset_powerset_nnc_polyhedron -> int
val ppl_Pointset_Powerset_NNC_Polyhedron_affine_dimension:
  pointset_powerset_nnc_polyhedron -> int




val ppl_Pointset_Powerset_NNC_Polyhedron_relation_with_constraint:
  pointset_powerset_nnc_polyhedron -> linear_constraint
  -> poly_con_relation list

val ppl_Pointset_Powerset_NNC_Polyhedron_relation_with_generator:
  pointset_powerset_nnc_polyhedron -> linear_generator
  -> poly_gen_relation list

val ppl_Pointset_Powerset_NNC_Polyhedron_relation_with_congruence:
  pointset_powerset_nnc_polyhedron -> linear_congruence
  -> poly_con_relation list





val ppl_Pointset_Powerset_NNC_Polyhedron_is_empty:
  pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_is_universe:
  pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_is_bounded:
  pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_contains_integer_point:
  pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_is_topologically_closed:
  pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_is_discrete:
  pointset_powerset_nnc_polyhedron -> bool





val ppl_Pointset_Powerset_NNC_Polyhedron_topological_closure_assign:
  pointset_powerset_nnc_polyhedron -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_pairwise_reduce:
  pointset_powerset_nnc_polyhedron -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_omega_reduce:
  pointset_powerset_nnc_polyhedron -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_bounds_from_above:
  pointset_powerset_nnc_polyhedron -> linear_expression -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_bounds_from_below:
  pointset_powerset_nnc_polyhedron -> linear_expression -> bool





val ppl_Pointset_Powerset_NNC_Polyhedron_maximize:
  pointset_powerset_nnc_polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Pointset_Powerset_NNC_Polyhedron_minimize:
  pointset_powerset_nnc_polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Pointset_Powerset_NNC_Polyhedron_maximize_with_point:
  pointset_powerset_nnc_polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Pointset_Powerset_NNC_Polyhedron_minimize_with_point:
  pointset_powerset_nnc_polyhedron -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Pointset_Powerset_NNC_Polyhedron_contains_Pointset_Powerset_NNC_Polyhedron:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_strictly_contains_Pointset_Powerset_NNC_Polyhedron:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_is_disjoint_from_Pointset_Powerset_NNC_Polyhedron:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_geometrically_covers_Pointset_Powerset_NNC_Polyhedron:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> bool

val ppl_Pointset_Powerset_NNC_Polyhedron_geometrically_equals_Pointset_Powerset_NNC_Polyhedron:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> bool





val ppl_Pointset_Powerset_NNC_Polyhedron_equals_Pointset_Powerset_NNC_Polyhedron:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> bool




val ppl_Pointset_Powerset_NNC_Polyhedron_OK:
  pointset_powerset_nnc_polyhedron -> bool



val ppl_Pointset_Powerset_NNC_Polyhedron_add_constraint:
  pointset_powerset_nnc_polyhedron -> linear_constraint -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_add_congruence:
  pointset_powerset_nnc_polyhedron -> linear_congruence -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_add_constraints:
  pointset_powerset_nnc_polyhedron -> constraint_system -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_add_congruences:
  pointset_powerset_nnc_polyhedron -> congruence_system -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_refine_with_constraint:
  pointset_powerset_nnc_polyhedron -> linear_constraint -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_refine_with_congruence:
  pointset_powerset_nnc_polyhedron -> linear_congruence -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_refine_with_constraints:
  pointset_powerset_nnc_polyhedron -> constraint_system -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_refine_with_congruences:
  pointset_powerset_nnc_polyhedron -> congruence_system -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_intersection_assign:
   pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_upper_bound_assign:
   pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_difference_assign:
   pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_concatenate_assign:
   pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_time_elapse_assign:
   pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_upper_bound_assign_if_exact:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> bool





val ppl_Pointset_Powerset_NNC_Polyhedron_simplify_using_context_assign:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> bool




val ppl_Pointset_Powerset_NNC_Polyhedron_constrains:
  pointset_powerset_nnc_polyhedron -> int -> bool




val ppl_Pointset_Powerset_NNC_Polyhedron_unconstrain_space_dimension:
  pointset_powerset_nnc_polyhedron -> int -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_unconstrain_space_dimensions:
  pointset_powerset_nnc_polyhedron -> int list -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_affine_image:
  pointset_powerset_nnc_polyhedron -> int -> linear_expression -> Z.t -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_affine_preimage:
  pointset_powerset_nnc_polyhedron -> int -> linear_expression -> Z.t -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_bounded_affine_image:
  pointset_powerset_nnc_polyhedron -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Pointset_Powerset_NNC_Polyhedron_bounded_affine_preimage:
  pointset_powerset_nnc_polyhedron -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_generalized_affine_image:
  pointset_powerset_nnc_polyhedron -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_generalized_affine_preimage:
  pointset_powerset_nnc_polyhedron -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_generalized_affine_image_lhs_rhs:
  pointset_powerset_nnc_polyhedron -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_generalized_affine_preimage_lhs_rhs:
  pointset_powerset_nnc_polyhedron -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_add_space_dimensions_and_embed:
  pointset_powerset_nnc_polyhedron -> int -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_add_space_dimensions_and_project:
  pointset_powerset_nnc_polyhedron -> int -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_remove_space_dimensions:
  pointset_powerset_nnc_polyhedron -> int list -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_remove_higher_space_dimensions:
  pointset_powerset_nnc_polyhedron -> int -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_expand_space_dimension:
  pointset_powerset_nnc_polyhedron -> int -> int -> unit



val ppl_Pointset_Powerset_NNC_Polyhedron_fold_space_dimensions:
  pointset_powerset_nnc_polyhedron -> int list -> int -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_map_space_dimensions:
  pointset_powerset_nnc_polyhedron -> (int*int) list -> unit



val ppl_Pointset_Powerset_NNC_Polyhedron_drop_some_non_integer_points:
  pointset_powerset_nnc_polyhedron -> complexity_class -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_drop_some_non_integer_points_2:
  pointset_powerset_nnc_polyhedron -> int list -> complexity_class -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_ascii_dump:
  pointset_powerset_nnc_polyhedron -> string




val ppl_Pointset_Powerset_NNC_Polyhedron_external_memory_in_bytes:
  pointset_powerset_nnc_polyhedron  -> int

val ppl_Pointset_Powerset_NNC_Polyhedron_total_memory_in_bytes:
  pointset_powerset_nnc_polyhedron  -> int

val ppl_Pointset_Powerset_NNC_Polyhedron_size:
  pointset_powerset_nnc_polyhedron  -> int





type pointset_powerset_nnc_polyhedron_iterator

val ppl_new_Pointset_Powerset_NNC_Polyhedron_iterator_from_iterator:
  pointset_powerset_nnc_polyhedron_iterator ->  pointset_powerset_nnc_polyhedron_iterator




val ppl_Pointset_Powerset_NNC_Polyhedron_begin_iterator:
  pointset_powerset_nnc_polyhedron  -> pointset_powerset_nnc_polyhedron_iterator

val ppl_Pointset_Powerset_NNC_Polyhedron_end_iterator:
  pointset_powerset_nnc_polyhedron  -> pointset_powerset_nnc_polyhedron_iterator





val ppl_Pointset_Powerset_NNC_Polyhedron_iterator_equals_iterator:
  pointset_powerset_nnc_polyhedron_iterator ->  pointset_powerset_nnc_polyhedron_iterator -> bool




val ppl_Pointset_Powerset_NNC_Polyhedron_increment_iterator:
  pointset_powerset_nnc_polyhedron_iterator -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_decrement_iterator:
  pointset_powerset_nnc_polyhedron_iterator -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_get_disjunct:
  pointset_powerset_nnc_polyhedron_iterator  ->  polyhedron







val ppl_Pointset_Powerset_NNC_Polyhedron_add_disjunct:
  pointset_powerset_nnc_polyhedron  ->  polyhedron -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_drop_disjunct:
  pointset_powerset_nnc_polyhedron  ->  pointset_powerset_nnc_polyhedron_iterator -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_drop_disjuncts:
  pointset_powerset_nnc_polyhedron  ->  pointset_powerset_nnc_polyhedron_iterator -> pointset_powerset_nnc_polyhedron_iterator -> unit




val ppl_Pointset_Powerset_NNC_Polyhedron_BHZ03_BHRZ03_BHRZ03_widening_assign:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_BHZ03_H79_H79_widening_assign:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> unit





val ppl_Pointset_Powerset_NNC_Polyhedron_BGP99_BHRZ03_extrapolation_assign:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> int -> unit

val ppl_Pointset_Powerset_NNC_Polyhedron_BGP99_H79_extrapolation_assign:
  pointset_powerset_nnc_polyhedron -> pointset_powerset_nnc_polyhedron -> int -> unit








val ppl_new_Double_Box_from_space_dimension:
  int -> degenerate_element -> double_box





val ppl_new_Double_Box_from_C_Polyhedron:
  polyhedron -> double_box

val ppl_new_Double_Box_from_NNC_Polyhedron:
  polyhedron -> double_box

val ppl_new_Double_Box_from_Grid:
  grid -> double_box

val ppl_new_Double_Box_from_Rational_Box:
  rational_box -> double_box

val ppl_new_Double_Box_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> double_box

val ppl_new_Double_Box_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> double_box

val ppl_new_Double_Box_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> double_box

val ppl_new_Double_Box_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> double_box

val ppl_new_Double_Box_from_Double_Box:
  double_box -> double_box

val ppl_new_Double_Box_from_BD_Shape_double:
  bd_shape_double -> double_box

val ppl_new_Double_Box_from_Octagonal_Shape_double:
  octagonal_shape_double -> double_box






val ppl_new_Double_Box_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> double_box

val ppl_new_Double_Box_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> double_box

val ppl_new_Double_Box_from_Grid_with_complexity:
  grid -> complexity_class -> double_box

val ppl_new_Double_Box_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> double_box

val ppl_new_Double_Box_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> double_box

val ppl_new_Double_Box_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> double_box

val ppl_new_Double_Box_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> double_box

val ppl_new_Double_Box_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> double_box

val ppl_new_Double_Box_from_Double_Box_with_complexity:
  double_box -> complexity_class -> double_box

val ppl_new_Double_Box_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> double_box

val ppl_new_Double_Box_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> double_box






val ppl_new_Double_Box_from_constraints:
  constraint_system -> double_box

val ppl_new_Double_Box_from_congruences:
  congruence_system -> double_box

val ppl_new_Double_Box_from_generators:
  generator_system -> double_box






val ppl_Double_Box_swap:
  double_box -> double_box -> unit




val ppl_Double_Box_space_dimension:
  double_box -> int
val ppl_Double_Box_affine_dimension:
  double_box -> int




val ppl_Double_Box_relation_with_constraint:
  double_box -> linear_constraint
  -> poly_con_relation list

val ppl_Double_Box_relation_with_generator:
  double_box -> linear_generator
  -> poly_gen_relation list

val ppl_Double_Box_relation_with_congruence:
  double_box -> linear_congruence
  -> poly_con_relation list





val ppl_Double_Box_get_constraints:
  double_box -> constraint_system

val ppl_Double_Box_get_congruences:
  double_box -> congruence_system





val ppl_Double_Box_get_minimized_constraints:
  double_box -> constraint_system

val ppl_Double_Box_get_minimized_congruences:
  double_box -> congruence_system





val ppl_Double_Box_is_empty:
  double_box -> bool

val ppl_Double_Box_is_universe:
  double_box -> bool

val ppl_Double_Box_is_bounded:
  double_box -> bool

val ppl_Double_Box_contains_integer_point:
  double_box -> bool

val ppl_Double_Box_is_topologically_closed:
  double_box -> bool

val ppl_Double_Box_is_discrete:
  double_box -> bool





val ppl_Double_Box_topological_closure_assign:
  double_box -> unit





val ppl_Double_Box_bounds_from_above:
  double_box -> linear_expression -> bool

val ppl_Double_Box_bounds_from_below:
  double_box -> linear_expression -> bool





val ppl_Double_Box_has_upper_bound:
  double_box -> int
  -> bool * Z.t * Z.t * bool

val ppl_Double_Box_has_lower_bound:
  double_box -> int
  -> bool * Z.t * Z.t * bool





val ppl_Double_Box_maximize:
  double_box -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Double_Box_minimize:
  double_box -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Double_Box_maximize_with_point:
  double_box -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Double_Box_minimize_with_point:
  double_box -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Double_Box_frequency:
  double_box -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_Double_Box_contains_Double_Box:
  double_box -> double_box -> bool

val ppl_Double_Box_strictly_contains_Double_Box:
  double_box -> double_box -> bool

val ppl_Double_Box_is_disjoint_from_Double_Box:
  double_box -> double_box -> bool





val ppl_Double_Box_equals_Double_Box:
  double_box -> double_box -> bool




val ppl_Double_Box_OK:
  double_box -> bool



val ppl_Double_Box_add_constraint:
  double_box -> linear_constraint -> unit

val ppl_Double_Box_add_congruence:
  double_box -> linear_congruence -> unit





val ppl_Double_Box_add_constraints:
  double_box -> constraint_system -> unit

val ppl_Double_Box_add_congruences:
  double_box -> congruence_system -> unit





val ppl_Double_Box_refine_with_constraint:
  double_box -> linear_constraint -> unit

val ppl_Double_Box_refine_with_congruence:
  double_box -> linear_congruence -> unit





val ppl_Double_Box_refine_with_constraints:
  double_box -> constraint_system -> unit

val ppl_Double_Box_refine_with_congruences:
  double_box -> congruence_system -> unit





val ppl_Double_Box_intersection_assign:
   double_box -> double_box -> unit

val ppl_Double_Box_upper_bound_assign:
   double_box -> double_box -> unit

val ppl_Double_Box_difference_assign:
   double_box -> double_box -> unit

val ppl_Double_Box_concatenate_assign:
   double_box -> double_box -> unit

val ppl_Double_Box_time_elapse_assign:
   double_box -> double_box -> unit





val ppl_Double_Box_upper_bound_assign_if_exact:
  double_box -> double_box -> bool





val ppl_Double_Box_simplify_using_context_assign:
  double_box -> double_box -> bool




val ppl_Double_Box_constrains:
  double_box -> int -> bool




val ppl_Double_Box_unconstrain_space_dimension:
  double_box -> int -> unit




val ppl_Double_Box_unconstrain_space_dimensions:
  double_box -> int list -> unit




val ppl_Double_Box_affine_image:
  double_box -> int -> linear_expression -> Z.t -> unit

val ppl_Double_Box_affine_preimage:
  double_box -> int -> linear_expression -> Z.t -> unit





val ppl_Double_Box_bounded_affine_image:
  double_box -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Double_Box_bounded_affine_preimage:
  double_box -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Double_Box_generalized_affine_image:
  double_box -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Double_Box_generalized_affine_preimage:
  double_box -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Double_Box_generalized_affine_image_lhs_rhs:
  double_box -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Double_Box_generalized_affine_preimage_lhs_rhs:
  double_box -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Double_Box_add_space_dimensions_and_embed:
  double_box -> int -> unit

val ppl_Double_Box_add_space_dimensions_and_project:
  double_box -> int -> unit





val ppl_Double_Box_remove_space_dimensions:
  double_box -> int list -> unit




val ppl_Double_Box_remove_higher_space_dimensions:
  double_box -> int -> unit




val ppl_Double_Box_expand_space_dimension:
  double_box -> int -> int -> unit



val ppl_Double_Box_fold_space_dimensions:
  double_box -> int list -> int -> unit




val ppl_Double_Box_map_space_dimensions:
  double_box -> (int*int) list -> unit



val ppl_Double_Box_drop_some_non_integer_points:
  double_box -> complexity_class -> unit




val ppl_Double_Box_drop_some_non_integer_points_2:
  double_box -> int list -> complexity_class -> unit




val ppl_Double_Box_ascii_dump:
  double_box -> string




val ppl_Double_Box_external_memory_in_bytes:
  double_box  -> int

val ppl_Double_Box_total_memory_in_bytes:
  double_box  -> int





val ppl_Double_Box_CC76_widening_assign_with_tokens:
  double_box -> double_box -> int -> int





val ppl_Double_Box_CC76_widening_assign:
  double_box -> double_box -> unit





val ppl_Double_Box_widening_assign_with_tokens:
  double_box -> double_box -> int -> int




val ppl_Double_Box_widening_assign:
  double_box -> double_box -> unit




val ppl_Double_Box_limited_CC76_extrapolation_assign_with_tokens:
  double_box -> double_box
  -> constraint_system -> int -> int







val ppl_Double_Box_limited_CC76_extrapolation_assign:
  double_box -> double_box
  -> constraint_system -> unit







val ppl_Double_Box_CC76_narrowing_assign:
  double_box  -> double_box -> unit





val ppl_Double_Box_linear_partition:
  double_box -> double_box -> double_box * pointset_powerset_nnc_polyhedron





val ppl_Double_Box_wrap_assign: double_box
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_Double_Box:
  double_box -> bool

val ppl_termination_test_PR_Double_Box:
  double_box -> bool






val ppl_one_affine_ranking_function_MS_Double_Box:
  double_box -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Double_Box:
  double_box -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Double_Box:
  double_box -> polyhedron

val ppl_all_affine_ranking_functions_PR_Double_Box:
  double_box -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Double_Box:
  double_box -> polyhedron * polyhedron





val ppl_termination_test_MS_Double_Box_2:
  double_box -> double_box -> bool

val ppl_termination_test_PR_Double_Box_2:
  double_box -> double_box -> bool






val ppl_one_affine_ranking_function_MS_Double_Box_2:
  double_box -> double_box -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Double_Box_2:
  double_box -> double_box -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Double_Box_2:
  double_box -> double_box -> polyhedron

val ppl_all_affine_ranking_functions_PR_Double_Box_2:
  double_box -> double_box -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Double_Box_2:
  double_box -> double_box -> polyhedron * polyhedron








val ppl_new_BD_Shape_double_from_space_dimension:
  int -> degenerate_element -> bd_shape_double





val ppl_new_BD_Shape_double_from_C_Polyhedron:
  polyhedron -> bd_shape_double

val ppl_new_BD_Shape_double_from_NNC_Polyhedron:
  polyhedron -> bd_shape_double

val ppl_new_BD_Shape_double_from_Grid:
  grid -> bd_shape_double

val ppl_new_BD_Shape_double_from_Rational_Box:
  rational_box -> bd_shape_double

val ppl_new_BD_Shape_double_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Double_Box:
  double_box -> bd_shape_double

val ppl_new_BD_Shape_double_from_BD_Shape_double:
  bd_shape_double -> bd_shape_double

val ppl_new_BD_Shape_double_from_Octagonal_Shape_double:
  octagonal_shape_double -> bd_shape_double






val ppl_new_BD_Shape_double_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Grid_with_complexity:
  grid -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Double_Box_with_complexity:
  double_box -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> bd_shape_double

val ppl_new_BD_Shape_double_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> bd_shape_double






val ppl_new_BD_Shape_double_from_constraints:
  constraint_system -> bd_shape_double

val ppl_new_BD_Shape_double_from_congruences:
  congruence_system -> bd_shape_double

val ppl_new_BD_Shape_double_from_generators:
  generator_system -> bd_shape_double






val ppl_BD_Shape_double_swap:
  bd_shape_double -> bd_shape_double -> unit




val ppl_BD_Shape_double_space_dimension:
  bd_shape_double -> int
val ppl_BD_Shape_double_affine_dimension:
  bd_shape_double -> int




val ppl_BD_Shape_double_relation_with_constraint:
  bd_shape_double -> linear_constraint
  -> poly_con_relation list

val ppl_BD_Shape_double_relation_with_generator:
  bd_shape_double -> linear_generator
  -> poly_gen_relation list

val ppl_BD_Shape_double_relation_with_congruence:
  bd_shape_double -> linear_congruence
  -> poly_con_relation list





val ppl_BD_Shape_double_get_constraints:
  bd_shape_double -> constraint_system

val ppl_BD_Shape_double_get_congruences:
  bd_shape_double -> congruence_system





val ppl_BD_Shape_double_get_minimized_constraints:
  bd_shape_double -> constraint_system

val ppl_BD_Shape_double_get_minimized_congruences:
  bd_shape_double -> congruence_system





val ppl_BD_Shape_double_is_empty:
  bd_shape_double -> bool

val ppl_BD_Shape_double_is_universe:
  bd_shape_double -> bool

val ppl_BD_Shape_double_is_bounded:
  bd_shape_double -> bool

val ppl_BD_Shape_double_contains_integer_point:
  bd_shape_double -> bool

val ppl_BD_Shape_double_is_topologically_closed:
  bd_shape_double -> bool

val ppl_BD_Shape_double_is_discrete:
  bd_shape_double -> bool





val ppl_BD_Shape_double_topological_closure_assign:
  bd_shape_double -> unit





val ppl_BD_Shape_double_bounds_from_above:
  bd_shape_double -> linear_expression -> bool

val ppl_BD_Shape_double_bounds_from_below:
  bd_shape_double -> linear_expression -> bool





val ppl_BD_Shape_double_maximize:
  bd_shape_double -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_BD_Shape_double_minimize:
  bd_shape_double -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_BD_Shape_double_maximize_with_point:
  bd_shape_double -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_BD_Shape_double_minimize_with_point:
  bd_shape_double -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_BD_Shape_double_frequency:
  bd_shape_double -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_BD_Shape_double_contains_BD_Shape_double:
  bd_shape_double -> bd_shape_double -> bool

val ppl_BD_Shape_double_strictly_contains_BD_Shape_double:
  bd_shape_double -> bd_shape_double -> bool

val ppl_BD_Shape_double_is_disjoint_from_BD_Shape_double:
  bd_shape_double -> bd_shape_double -> bool





val ppl_BD_Shape_double_equals_BD_Shape_double:
  bd_shape_double -> bd_shape_double -> bool




val ppl_BD_Shape_double_OK:
  bd_shape_double -> bool



val ppl_BD_Shape_double_add_constraint:
  bd_shape_double -> linear_constraint -> unit

val ppl_BD_Shape_double_add_congruence:
  bd_shape_double -> linear_congruence -> unit





val ppl_BD_Shape_double_add_constraints:
  bd_shape_double -> constraint_system -> unit

val ppl_BD_Shape_double_add_congruences:
  bd_shape_double -> congruence_system -> unit





val ppl_BD_Shape_double_refine_with_constraint:
  bd_shape_double -> linear_constraint -> unit

val ppl_BD_Shape_double_refine_with_congruence:
  bd_shape_double -> linear_congruence -> unit





val ppl_BD_Shape_double_refine_with_constraints:
  bd_shape_double -> constraint_system -> unit

val ppl_BD_Shape_double_refine_with_congruences:
  bd_shape_double -> congruence_system -> unit





val ppl_BD_Shape_double_intersection_assign:
   bd_shape_double -> bd_shape_double -> unit

val ppl_BD_Shape_double_upper_bound_assign:
   bd_shape_double -> bd_shape_double -> unit

val ppl_BD_Shape_double_difference_assign:
   bd_shape_double -> bd_shape_double -> unit

val ppl_BD_Shape_double_concatenate_assign:
   bd_shape_double -> bd_shape_double -> unit

val ppl_BD_Shape_double_time_elapse_assign:
   bd_shape_double -> bd_shape_double -> unit





val ppl_BD_Shape_double_upper_bound_assign_if_exact:
  bd_shape_double -> bd_shape_double -> bool





val ppl_BD_Shape_double_simplify_using_context_assign:
  bd_shape_double -> bd_shape_double -> bool




val ppl_BD_Shape_double_constrains:
  bd_shape_double -> int -> bool




val ppl_BD_Shape_double_unconstrain_space_dimension:
  bd_shape_double -> int -> unit




val ppl_BD_Shape_double_unconstrain_space_dimensions:
  bd_shape_double -> int list -> unit




val ppl_BD_Shape_double_affine_image:
  bd_shape_double -> int -> linear_expression -> Z.t -> unit

val ppl_BD_Shape_double_affine_preimage:
  bd_shape_double -> int -> linear_expression -> Z.t -> unit





val ppl_BD_Shape_double_bounded_affine_image:
  bd_shape_double -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_BD_Shape_double_bounded_affine_preimage:
  bd_shape_double -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_BD_Shape_double_generalized_affine_image:
  bd_shape_double -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_BD_Shape_double_generalized_affine_preimage:
  bd_shape_double -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_BD_Shape_double_generalized_affine_image_lhs_rhs:
  bd_shape_double -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_BD_Shape_double_generalized_affine_preimage_lhs_rhs:
  bd_shape_double -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_BD_Shape_double_add_space_dimensions_and_embed:
  bd_shape_double -> int -> unit

val ppl_BD_Shape_double_add_space_dimensions_and_project:
  bd_shape_double -> int -> unit





val ppl_BD_Shape_double_remove_space_dimensions:
  bd_shape_double -> int list -> unit




val ppl_BD_Shape_double_remove_higher_space_dimensions:
  bd_shape_double -> int -> unit




val ppl_BD_Shape_double_expand_space_dimension:
  bd_shape_double -> int -> int -> unit



val ppl_BD_Shape_double_fold_space_dimensions:
  bd_shape_double -> int list -> int -> unit




val ppl_BD_Shape_double_map_space_dimensions:
  bd_shape_double -> (int*int) list -> unit



val ppl_BD_Shape_double_drop_some_non_integer_points:
  bd_shape_double -> complexity_class -> unit




val ppl_BD_Shape_double_drop_some_non_integer_points_2:
  bd_shape_double -> int list -> complexity_class -> unit




val ppl_BD_Shape_double_ascii_dump:
  bd_shape_double -> string




val ppl_BD_Shape_double_external_memory_in_bytes:
  bd_shape_double  -> int

val ppl_BD_Shape_double_total_memory_in_bytes:
  bd_shape_double  -> int





val ppl_BD_Shape_double_BHMZ05_widening_assign_with_tokens:
  bd_shape_double -> bd_shape_double -> int -> int

val ppl_BD_Shape_double_H79_widening_assign_with_tokens:
  bd_shape_double -> bd_shape_double -> int -> int





val ppl_BD_Shape_double_BHMZ05_widening_assign:
  bd_shape_double -> bd_shape_double -> unit

val ppl_BD_Shape_double_H79_widening_assign:
  bd_shape_double -> bd_shape_double -> unit





val ppl_BD_Shape_double_widening_assign_with_tokens:
  bd_shape_double -> bd_shape_double -> int -> int




val ppl_BD_Shape_double_widening_assign:
  bd_shape_double -> bd_shape_double -> unit




val ppl_BD_Shape_double_limited_BHMZ05_extrapolation_assign_with_tokens:
  bd_shape_double -> bd_shape_double
  -> constraint_system -> int -> int

val ppl_BD_Shape_double_limited_H79_extrapolation_assign_with_tokens:
  bd_shape_double -> bd_shape_double
  -> constraint_system -> int -> int

val ppl_BD_Shape_double_limited_CC76_extrapolation_assign_with_tokens:
  bd_shape_double -> bd_shape_double
  -> constraint_system -> int -> int







val ppl_BD_Shape_double_limited_BHMZ05_extrapolation_assign:
  bd_shape_double -> bd_shape_double
  -> constraint_system -> unit

val ppl_BD_Shape_double_limited_H79_extrapolation_assign:
  bd_shape_double -> bd_shape_double
  -> constraint_system -> unit

val ppl_BD_Shape_double_limited_CC76_extrapolation_assign:
  bd_shape_double -> bd_shape_double
  -> constraint_system -> unit







val ppl_BD_Shape_double_CC76_extrapolation_assign_with_tokens:
  bd_shape_double  -> bd_shape_double -> int -> int





val ppl_BD_Shape_double_CC76_extrapolation_assign:
  bd_shape_double  -> bd_shape_double -> unit





val ppl_BD_Shape_double_CC76_narrowing_assign:
  bd_shape_double  -> bd_shape_double -> unit





val ppl_BD_Shape_double_linear_partition:
  bd_shape_double -> bd_shape_double -> bd_shape_double * pointset_powerset_nnc_polyhedron





val ppl_BD_Shape_double_wrap_assign: bd_shape_double
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_BD_Shape_double:
  bd_shape_double -> bool

val ppl_termination_test_PR_BD_Shape_double:
  bd_shape_double -> bool






val ppl_one_affine_ranking_function_MS_BD_Shape_double:
  bd_shape_double -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_BD_Shape_double:
  bd_shape_double -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_BD_Shape_double:
  bd_shape_double -> polyhedron

val ppl_all_affine_ranking_functions_PR_BD_Shape_double:
  bd_shape_double -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_BD_Shape_double:
  bd_shape_double -> polyhedron * polyhedron





val ppl_termination_test_MS_BD_Shape_double_2:
  bd_shape_double -> bd_shape_double -> bool

val ppl_termination_test_PR_BD_Shape_double_2:
  bd_shape_double -> bd_shape_double -> bool






val ppl_one_affine_ranking_function_MS_BD_Shape_double_2:
  bd_shape_double -> bd_shape_double -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_BD_Shape_double_2:
  bd_shape_double -> bd_shape_double -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_BD_Shape_double_2:
  bd_shape_double -> bd_shape_double -> polyhedron

val ppl_all_affine_ranking_functions_PR_BD_Shape_double_2:
  bd_shape_double -> bd_shape_double -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_BD_Shape_double_2:
  bd_shape_double -> bd_shape_double -> polyhedron * polyhedron








val ppl_new_Octagonal_Shape_double_from_space_dimension:
  int -> degenerate_element -> octagonal_shape_double





val ppl_new_Octagonal_Shape_double_from_C_Polyhedron:
  polyhedron -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_NNC_Polyhedron:
  polyhedron -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Grid:
  grid -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Rational_Box:
  rational_box -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_BD_Shape_mpz_class:
  bd_shape_mpz_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_BD_Shape_mpq_class:
  bd_shape_mpq_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Octagonal_Shape_mpz_class:
  octagonal_shape_mpz_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Octagonal_Shape_mpq_class:
  octagonal_shape_mpq_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Double_Box:
  double_box -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_BD_Shape_double:
  bd_shape_double -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Octagonal_Shape_double:
  octagonal_shape_double -> octagonal_shape_double






val ppl_new_Octagonal_Shape_double_from_C_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_NNC_Polyhedron_with_complexity:
  polyhedron -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Grid_with_complexity:
  grid -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Rational_Box_with_complexity:
  rational_box -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_BD_Shape_mpz_class_with_complexity:
  bd_shape_mpz_class -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_BD_Shape_mpq_class_with_complexity:
  bd_shape_mpq_class -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Octagonal_Shape_mpz_class_with_complexity:
  octagonal_shape_mpz_class -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Octagonal_Shape_mpq_class_with_complexity:
  octagonal_shape_mpq_class -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Double_Box_with_complexity:
  double_box -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_BD_Shape_double_with_complexity:
  bd_shape_double -> complexity_class -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_Octagonal_Shape_double_with_complexity:
  octagonal_shape_double -> complexity_class -> octagonal_shape_double






val ppl_new_Octagonal_Shape_double_from_constraints:
  constraint_system -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_congruences:
  congruence_system -> octagonal_shape_double

val ppl_new_Octagonal_Shape_double_from_generators:
  generator_system -> octagonal_shape_double






val ppl_Octagonal_Shape_double_swap:
  octagonal_shape_double -> octagonal_shape_double -> unit




val ppl_Octagonal_Shape_double_space_dimension:
  octagonal_shape_double -> int
val ppl_Octagonal_Shape_double_affine_dimension:
  octagonal_shape_double -> int




val ppl_Octagonal_Shape_double_relation_with_constraint:
  octagonal_shape_double -> linear_constraint
  -> poly_con_relation list

val ppl_Octagonal_Shape_double_relation_with_generator:
  octagonal_shape_double -> linear_generator
  -> poly_gen_relation list

val ppl_Octagonal_Shape_double_relation_with_congruence:
  octagonal_shape_double -> linear_congruence
  -> poly_con_relation list





val ppl_Octagonal_Shape_double_get_constraints:
  octagonal_shape_double -> constraint_system

val ppl_Octagonal_Shape_double_get_congruences:
  octagonal_shape_double -> congruence_system





val ppl_Octagonal_Shape_double_get_minimized_constraints:
  octagonal_shape_double -> constraint_system

val ppl_Octagonal_Shape_double_get_minimized_congruences:
  octagonal_shape_double -> congruence_system





val ppl_Octagonal_Shape_double_is_empty:
  octagonal_shape_double -> bool

val ppl_Octagonal_Shape_double_is_universe:
  octagonal_shape_double -> bool

val ppl_Octagonal_Shape_double_is_bounded:
  octagonal_shape_double -> bool

val ppl_Octagonal_Shape_double_contains_integer_point:
  octagonal_shape_double -> bool

val ppl_Octagonal_Shape_double_is_topologically_closed:
  octagonal_shape_double -> bool

val ppl_Octagonal_Shape_double_is_discrete:
  octagonal_shape_double -> bool





val ppl_Octagonal_Shape_double_topological_closure_assign:
  octagonal_shape_double -> unit





val ppl_Octagonal_Shape_double_bounds_from_above:
  octagonal_shape_double -> linear_expression -> bool

val ppl_Octagonal_Shape_double_bounds_from_below:
  octagonal_shape_double -> linear_expression -> bool





val ppl_Octagonal_Shape_double_maximize:
  octagonal_shape_double -> linear_expression
  -> bool * Z.t * Z.t * bool

val ppl_Octagonal_Shape_double_minimize:
  octagonal_shape_double -> linear_expression
  -> bool * Z.t * Z.t * bool





val ppl_Octagonal_Shape_double_maximize_with_point:
  octagonal_shape_double -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator

val ppl_Octagonal_Shape_double_minimize_with_point:
  octagonal_shape_double -> linear_expression
  -> bool * Z.t * Z.t * bool * linear_generator





val ppl_Octagonal_Shape_double_frequency:
  octagonal_shape_double -> linear_expression
  -> bool * Z.t * Z.t * Z.t * Z.t




val ppl_Octagonal_Shape_double_contains_Octagonal_Shape_double:
  octagonal_shape_double -> octagonal_shape_double -> bool

val ppl_Octagonal_Shape_double_strictly_contains_Octagonal_Shape_double:
  octagonal_shape_double -> octagonal_shape_double -> bool

val ppl_Octagonal_Shape_double_is_disjoint_from_Octagonal_Shape_double:
  octagonal_shape_double -> octagonal_shape_double -> bool





val ppl_Octagonal_Shape_double_equals_Octagonal_Shape_double:
  octagonal_shape_double -> octagonal_shape_double -> bool




val ppl_Octagonal_Shape_double_OK:
  octagonal_shape_double -> bool



val ppl_Octagonal_Shape_double_add_constraint:
  octagonal_shape_double -> linear_constraint -> unit

val ppl_Octagonal_Shape_double_add_congruence:
  octagonal_shape_double -> linear_congruence -> unit





val ppl_Octagonal_Shape_double_add_constraints:
  octagonal_shape_double -> constraint_system -> unit

val ppl_Octagonal_Shape_double_add_congruences:
  octagonal_shape_double -> congruence_system -> unit





val ppl_Octagonal_Shape_double_refine_with_constraint:
  octagonal_shape_double -> linear_constraint -> unit

val ppl_Octagonal_Shape_double_refine_with_congruence:
  octagonal_shape_double -> linear_congruence -> unit





val ppl_Octagonal_Shape_double_refine_with_constraints:
  octagonal_shape_double -> constraint_system -> unit

val ppl_Octagonal_Shape_double_refine_with_congruences:
  octagonal_shape_double -> congruence_system -> unit





val ppl_Octagonal_Shape_double_intersection_assign:
   octagonal_shape_double -> octagonal_shape_double -> unit

val ppl_Octagonal_Shape_double_upper_bound_assign:
   octagonal_shape_double -> octagonal_shape_double -> unit

val ppl_Octagonal_Shape_double_difference_assign:
   octagonal_shape_double -> octagonal_shape_double -> unit

val ppl_Octagonal_Shape_double_concatenate_assign:
   octagonal_shape_double -> octagonal_shape_double -> unit

val ppl_Octagonal_Shape_double_time_elapse_assign:
   octagonal_shape_double -> octagonal_shape_double -> unit





val ppl_Octagonal_Shape_double_upper_bound_assign_if_exact:
  octagonal_shape_double -> octagonal_shape_double -> bool





val ppl_Octagonal_Shape_double_simplify_using_context_assign:
  octagonal_shape_double -> octagonal_shape_double -> bool




val ppl_Octagonal_Shape_double_constrains:
  octagonal_shape_double -> int -> bool




val ppl_Octagonal_Shape_double_unconstrain_space_dimension:
  octagonal_shape_double -> int -> unit




val ppl_Octagonal_Shape_double_unconstrain_space_dimensions:
  octagonal_shape_double -> int list -> unit




val ppl_Octagonal_Shape_double_affine_image:
  octagonal_shape_double -> int -> linear_expression -> Z.t -> unit

val ppl_Octagonal_Shape_double_affine_preimage:
  octagonal_shape_double -> int -> linear_expression -> Z.t -> unit





val ppl_Octagonal_Shape_double_bounded_affine_image:
  octagonal_shape_double -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit
val ppl_Octagonal_Shape_double_bounded_affine_preimage:
  octagonal_shape_double -> int
  -> linear_expression -> linear_expression
  -> Z.t -> unit




val ppl_Octagonal_Shape_double_generalized_affine_image:
  octagonal_shape_double -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit

val ppl_Octagonal_Shape_double_generalized_affine_preimage:
  octagonal_shape_double -> int -> relation_symbol -> linear_expression
  -> Z.t -> unit





val ppl_Octagonal_Shape_double_generalized_affine_image_lhs_rhs:
  octagonal_shape_double -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit

val ppl_Octagonal_Shape_double_generalized_affine_preimage_lhs_rhs:
  octagonal_shape_double -> linear_expression
  -> relation_symbol -> linear_expression
  -> unit





val ppl_Octagonal_Shape_double_add_space_dimensions_and_embed:
  octagonal_shape_double -> int -> unit

val ppl_Octagonal_Shape_double_add_space_dimensions_and_project:
  octagonal_shape_double -> int -> unit





val ppl_Octagonal_Shape_double_remove_space_dimensions:
  octagonal_shape_double -> int list -> unit




val ppl_Octagonal_Shape_double_remove_higher_space_dimensions:
  octagonal_shape_double -> int -> unit




val ppl_Octagonal_Shape_double_expand_space_dimension:
  octagonal_shape_double -> int -> int -> unit



val ppl_Octagonal_Shape_double_fold_space_dimensions:
  octagonal_shape_double -> int list -> int -> unit




val ppl_Octagonal_Shape_double_map_space_dimensions:
  octagonal_shape_double -> (int*int) list -> unit



val ppl_Octagonal_Shape_double_drop_some_non_integer_points:
  octagonal_shape_double -> complexity_class -> unit




val ppl_Octagonal_Shape_double_drop_some_non_integer_points_2:
  octagonal_shape_double -> int list -> complexity_class -> unit




val ppl_Octagonal_Shape_double_ascii_dump:
  octagonal_shape_double -> string




val ppl_Octagonal_Shape_double_external_memory_in_bytes:
  octagonal_shape_double  -> int

val ppl_Octagonal_Shape_double_total_memory_in_bytes:
  octagonal_shape_double  -> int





val ppl_Octagonal_Shape_double_BHMZ05_widening_assign_with_tokens:
  octagonal_shape_double -> octagonal_shape_double -> int -> int





val ppl_Octagonal_Shape_double_BHMZ05_widening_assign:
  octagonal_shape_double -> octagonal_shape_double -> unit





val ppl_Octagonal_Shape_double_widening_assign_with_tokens:
  octagonal_shape_double -> octagonal_shape_double -> int -> int




val ppl_Octagonal_Shape_double_widening_assign:
  octagonal_shape_double -> octagonal_shape_double -> unit




val ppl_Octagonal_Shape_double_limited_BHMZ05_extrapolation_assign_with_tokens:
  octagonal_shape_double -> octagonal_shape_double
  -> constraint_system -> int -> int

val ppl_Octagonal_Shape_double_limited_CC76_extrapolation_assign_with_tokens:
  octagonal_shape_double -> octagonal_shape_double
  -> constraint_system -> int -> int







val ppl_Octagonal_Shape_double_limited_BHMZ05_extrapolation_assign:
  octagonal_shape_double -> octagonal_shape_double
  -> constraint_system -> unit

val ppl_Octagonal_Shape_double_limited_CC76_extrapolation_assign:
  octagonal_shape_double -> octagonal_shape_double
  -> constraint_system -> unit







val ppl_Octagonal_Shape_double_CC76_extrapolation_assign_with_tokens:
  octagonal_shape_double  -> octagonal_shape_double -> int -> int





val ppl_Octagonal_Shape_double_CC76_extrapolation_assign:
  octagonal_shape_double  -> octagonal_shape_double -> unit





val ppl_Octagonal_Shape_double_CC76_narrowing_assign:
  octagonal_shape_double  -> octagonal_shape_double -> unit





val ppl_Octagonal_Shape_double_linear_partition:
  octagonal_shape_double -> octagonal_shape_double -> octagonal_shape_double * pointset_powerset_nnc_polyhedron





val ppl_Octagonal_Shape_double_wrap_assign: octagonal_shape_double
                             -> int list
                             -> bounded_integer_type_width
                             -> bounded_integer_type_representation
                             -> bounded_integer_type_overflow
                             -> constraint_system
                             -> int -> int -> unit




val ppl_termination_test_MS_Octagonal_Shape_double:
  octagonal_shape_double -> bool

val ppl_termination_test_PR_Octagonal_Shape_double:
  octagonal_shape_double -> bool






val ppl_one_affine_ranking_function_MS_Octagonal_Shape_double:
  octagonal_shape_double -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Octagonal_Shape_double:
  octagonal_shape_double -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Octagonal_Shape_double:
  octagonal_shape_double -> polyhedron

val ppl_all_affine_ranking_functions_PR_Octagonal_Shape_double:
  octagonal_shape_double -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Octagonal_Shape_double:
  octagonal_shape_double -> polyhedron * polyhedron





val ppl_termination_test_MS_Octagonal_Shape_double_2:
  octagonal_shape_double -> octagonal_shape_double -> bool

val ppl_termination_test_PR_Octagonal_Shape_double_2:
  octagonal_shape_double -> octagonal_shape_double -> bool






val ppl_one_affine_ranking_function_MS_Octagonal_Shape_double_2:
  octagonal_shape_double -> octagonal_shape_double -> bool * linear_generator

val ppl_one_affine_ranking_function_PR_Octagonal_Shape_double_2:
  octagonal_shape_double -> octagonal_shape_double -> bool * linear_generator






val ppl_all_affine_ranking_functions_MS_Octagonal_Shape_double_2:
  octagonal_shape_double -> octagonal_shape_double -> polyhedron

val ppl_all_affine_ranking_functions_PR_Octagonal_Shape_double_2:
  octagonal_shape_double -> octagonal_shape_double -> polyhedron






val ppl_all_affine_quasi_ranking_functions_MS_Octagonal_Shape_double_2:
  octagonal_shape_double -> octagonal_shape_double -> polyhedron * polyhedron






